# ==================================================================
# Gossamer Threads Module Library - http://gossamer-threads.com/
#
#   GT::CGI
#   Author  : Aki Mimoto
#   $Id: CGI.pm,v 1.93 2001/11/28 03:16:54 jagerman Exp $
# 
# Copyright (c) 2000 Gossamer Threads Inc.  All Rights Reserved.
# ==================================================================
#
# Description:
#   Implements a use CGI clone without all the overhead.
#

package GT::CGI;
# ===============================================================
    use strict;
    use GT::Base (':all');  # Imports $MOD_PERL, $SPEEDY and $PERSIST
    use vars qw/@ISA $DEBUG $VERSION $ATTRIBS $ERRORS $PRINTED_HEAD 
                $INPUT_SEPARATOR $FORM_PARSED $POST_MAX $CRLF %PARAMS %COOKIES $PARSE_BUTTONS/;

    $PARSE_BUTTONS      = 1;
    @ISA                = qw/GT::Base/;
    $DEBUG              = 0;
    $VERSION            = sprintf "%d.%03d", q$Revision: 1.93 $ =~ /(\d+)\.(\d+)/;
    $ATTRIBS            = {
                            'nph'             => 0,
                            'p'               => ''
                        };
    $ERRORS             = {
                            'INVALIDCOOKIE'   => "Invalid cookie passed to header: %s",
                            'INVALIDDATE'     => "Date '%s' is not a valid date format.",
                        };
    $PRINTED_HEAD       = 0;
    $FORM_PARSED        = 0;
    $CRLF               = "\015\012";
    $INPUT_SEPARATOR    = chr(0);
    %PARAMS             = ();
    %COOKIES            = ();

# Pre load our compiled if under mod_perl/speedy.
    if ($PERSIST) {
        read (GT::CGI::MultiPart::DATA, my $code, -s GT::CGI::MultiPart::DATA);
        eval "package GT::CGI::MultiPart;\n#line 1GT::CGI::MultiPart\n$code";
        $@ and die "GT::CGI => Unable to load multipart parser code: $@";
    }

sub import {
#--------------------------------------------------------------------------------
# You can pass in -no_parse_buttons to not have form keys parsed.
#
    if ( $_[1] and $_[1] eq '-no_parse_buttons' ) {
        $PARSE_BUTTONS = 0;
    }
}
    
sub load_data {
#--------------------------------------------------------------------------------
# Loads the form information into PARAMS. Data comes from either
# a multipart form, a GET Request, a POST request, or as arguments from command 
# line.
#
    my $self = shift;
    %PARAMS  = ();
    %COOKIES = ();

# Load form data.
    my $method         = defined $ENV{REQUEST_METHOD}   ? uc $ENV{REQUEST_METHOD} : '';
    my $content_length = defined $ENV{'CONTENT_LENGTH'} ? $ENV{'CONTENT_LENGTH'} : 0;

    if ($method eq 'GET') {
        $self->parse_str(defined $ENV{QUERY_STRING} ? $ENV{QUERY_STRING} : '');
    }
    elsif ($method eq 'POST') {
        if ($content_length) {
            if ((defined $ENV{CONTENT_TYPE} ? $ENV{CONTENT_TYPE} : '') =~ m,^multipart,) {

# The rest of the code is the multipart file upload code. Warning, do not
# change the package space (i.e. move GT::CGI::Cookie) without updating 
# sub parse_multipart().
                if (! defined &GT::CGI::MultiPart::init) { 
                    read (GT::CGI::MultiPart::DATA, my $code, -s GT::CGI::MultiPart::DATA);
                    eval "package GT::CGI::MultiPart;\n#line 1GT::CGI::MultiPart\n$code";
                    $@ and die "GT::CGI => Unable to load multipart parser code: $@";
                }
                GT::CGI::MultiPart->parse($self);
            }
            else {
                read (STDIN, my $data, $content_length, 0);
                $data =~ s/\r?\n/&/g;
                $self->parse_str($data);
            }
        }
    }
    else {
        my $data = join ("&", @ARGV );
        $self->parse_str($data);
    }

# Load cookies.
    if (defined $ENV{HTTP_COOKIE}) {
        %{$self->{cookies}} =  map { 
            /(.*)=(.*)/; 
            my $key = unescape($1);
            my $val = unescape($2);
            if ($val =~ /&/) {
                $val = [ split /&/ => $val ];
            }
            else {
                $val = [ $val ];
            }
            $key => $val;
        } split(/\;\s*/, $ENV{HTTP_COOKIE});
    }
    else {
        %{$self->{cookies}} = ();
    }

# If we are under mod_perl we let mod_perl know that it should call reset_param
# when a request is finished.
    $MOD_PERL and require Apache and $Apache::ServerStarting != 1 and Apache->request->register_cleanup(\&reset_env);
    $SPEEDY   and require CGI::SpeedyCGI and CGI::SpeedyCGI->new->register_cleanup (\&reset_env );

# Parse form buttons, allowing you to pass in name="foo=bar;a=b;c=d" as a name
# tag in the form.
    if ($PARSE_BUTTONS) {
        for (keys %{$self->{params}}) {
            if (index($_, '=') >= 0) {
                next if substr($_, -2) eq '.y';
                s/\.x$//;
                delete $self->{params}->{$_};
                for (split /[&;]/) {
                    /^([^=]+)=(.*)/ or next;
                    my ($key, $val) = (unescape($1), unescape($2));
                    # Need to remove cr's on windows.
                    if ($^O eq 'MSWin32') {
                        $key =~ s/\r\n/\n/g;
                        $val =~ s/\r\n/\n/g;
                    }
                    unshift @{$self->{params}{$key}}, $val;
                }
            }
        }
    }
    
# Save the data for caching
    for (keys %{$self->{params}}) {
        for my $p (@{$self->{params}->{$_}}) {
            push @{$PARAMS{$_}}, $p;
        }
    }
    for (keys %{$self->{cookies}}) {
        for my $p (@{$self->{cookies}->{$_}}) {
            push @{$COOKIES{$_}}, $p;
        }
    }
    $FORM_PARSED = 1;
}

sub class_new {
# --------------------------------------------------------------------------------
# Creates an object to be used for all class methods, this affects the global
# cookies and params.
#
    my $self = {};
    $self->{cookies} = \%COOKIES;
    $self->{params}  = \%PARAMS;
    for (keys %{$ATTRIBS}) { $self->{$_} = $ATTRIBS->{$_} }
    return bless $self => shift;
}

sub reset_env {
# --------------------------------------------------------------------------------
# Reset the global environment.
#
    %PARAMS         = ();
    %COOKIES        = ();
    $PRINTED_HEAD   = 0; 
    $FORM_PARSED    = 0;
    1;
}

sub init {
#--------------------------------------------------------------------------------
# Called from GT::Base when a new object is created.
#
    my $self = shift;

# If we are passed a single argument, then we load our data from
# the input.
    if (@_ == 1) {
        my $p = $_[0];
        if (ref $p eq 'GT::CGI') {
            $p = $p->query_string;
        }
        $self->parse_str($p ? "&$p" : "");
        if (defined $ENV{HTTP_COOKIE}) {
            %{$self->{cookies}} =  map { 
                /(.*)=(.*)/; 
                my $key = unescape($1);
                my $val = unescape($2);
                if ($val =~ /&/) {
                    $val = [ split /&/ => $val ];
                }
                else {
                    $val = [ $val ];
                }
                $key => $val;
            } split(/\;\s*/, $ENV{HTTP_COOKIE});
        }
        $FORM_PARSED = 1;
    }
    else {
        $self->set(@_) if (@_);

# If we have the form parsed, then we need to copy the data into self.
        if ($FORM_PARSED) {
            foreach my $k (keys %PARAMS) {
                foreach my $v (@{$PARAMS{$k}}) {
                    push @{$self->{params}->{$k}}, $v;
                }
            }
            foreach my $k (keys %COOKIES) {
                foreach my $v (@{$COOKIES{$k}}) {
                    push @{$self->{cookies}->{$k}}, $v;
                }
            }
        }    
    }
    return $self;
}

sub get_hash {
#--------------------------------------------------------------------------------
# Returns the parameters as a HASH, with multiple values glued together
# with $INPUT_SEPARATOR. You can pass in true if you want values joined together (default)
# or false if you want the values as an array reference.
#
    my $self = shift;
    $self = $self->class_new unless ref $self;
    $FORM_PARSED or $self->load_data();
    my $join = defined $_[0] ? $_[0] : 0;

    keys %{$self->{params}} or return {};

# Construct hash ref and return it
    my $opts = {};
    foreach (keys %{$self->{params}}) { 
        my @vals = @{$self->{params}->{$_}};
        if (@vals > 1) {
            $opts->{$_} = $join ? join ($INPUT_SEPARATOR, @vals) : \@vals;
        }
        else {
            $opts->{$_} = $vals[0];
        }
    }
    return $opts;
}

sub delete {
#--------------------------------------------------------------------------------
# Remove an element from the parameters.
#
    my ($self, $param) = @_;
    $self = $self->class_new unless ref $self;
    $FORM_PARSED or $self->load_data();
    exists $self->{params}->{$param} and return wantarray ? @{delete $self->{params}->{$param}} : (@{delete $self->{params}->{$param}})[0];
    return;
}

sub cookie {
#--------------------------------------------------------------------------------
# Creates a new cookie for the user, implemented just like CGI.pm.
#
    my $self = shift;        # Not used, don't care if it's self/class.;
    $self = $self->class_new unless ref $self;
    $FORM_PARSED or $self->load_data();
    my %data = ( @_ ) if ( @_ and @_ % 2 == 0 );
    if (@_ == 0) {          # Return keys.
        return keys %{$self->{cookies}};
    }
    elsif (@_ == 1) {       # Return value of param passed in.
        my $param = shift;
        return unless (defined ($param) and $self->{cookies}->{$param});
        return wantarray ? @{$self->{cookies}->{$param}} : $self->{cookies}->{$param}->[0];
    }
    elsif (@_ == 2) {
        return new GT::CGI::Cookie ( -name => $_[0], -value => $_[1] );
    }
    elsif (defined $data{'-value'}) {
        return new GT::CGI::Cookie( %data );
    }
    else {                  # Set parameter.
        my ($param, $value) = @_;
        $self->{cookies}->{$param} = (ref $value eq 'ARRAY' ? $value : [$value]);
    }
}

sub set {
#--------------------------------------------------------------------------------
# Let's you set a key/val parameter.
#
    my $self    = shift;
    $self = $self->class_new unless ref $self;
    $FORM_PARSED or $self->load_data();
    my $params  = $self->common_param(@_);

    foreach my $key (keys %{$params}) {
        $self->{params}->{$key} =  $$params{$key};
    }
    return {%{$self->{params}}};
}

sub param {
#--------------------------------------------------------------------------------
# Mimick CGI's param function for get/set.
#
    my $self = shift;        # Not used, don't care if it's self/class.;
    $self = $self->class_new unless ref $self;
    $FORM_PARSED or $self->load_data();
    if (@_ == 0) {          # Return keys.
        return keys %{$self->{params}};
    }
    elsif (@_ == 1) {       # Return value of param passed in.
        my $param = shift;
        return unless (defined ($param) and $self->{params}->{$param});
        return wantarray ? @{$self->{params}->{$param}} : $self->{params}->{$param}->[0];
    }
    else {                  # Set parameter.
        my ($param, $value) = @_;
        (ref $value eq 'ARRAY') ?
            ($self->{params}->{$param} = $value) : ($self->{params}->{$param} = [$value]);
    }
}

sub header {
#--------------------------------------------------------------------------------
# Mimick the header function.
#
    my $self    = shift;
    $self = $self->class_new unless ref $self;
    my %p       = (ref($_[0]) eq 'HASH') ? %{$_[0]} : ( @_ % 2 ) ? () : @_; 
    my @headers;

# Don't print headers twice unless -force'd.
    if ($PRINTED_HEAD and !$p{'-force'}) {
        return "";
    }
    delete $p{'-force'};

# Start by adding NPH headers if requested.
    if ($self->{'nph'} || $p{'-nph'}) {
        if ($p{'-url'}) {
            push @headers, "HTTP/1.0 302 Moved";
        }
        else {
            my $protocol = $ENV{SERVER_PROTOCOL} || 'HTTP/1.0';
            push @headers, "$protocol 200 OK" unless ($MOD_PERL);
        }
    }
    delete $p{'-nph'};

# Add any cookies, we accept either an array of cookies
# or a single cookie.
    my $add_date  = 0;
    my $cookies   = 0;
    my $container = defined($p{'-cookie'}) ? $p{'-cookie'} : '';
    if (ref($container) eq 'GT::CGI::Cookie') {
        my $c = $container->cookie_header;
        push @headers, $c;
        $add_date = 1;
        $cookies++;
    }
    elsif (ref $container eq 'ARRAY') {
        foreach my $cookie ( @$container ) {
            next unless (defined $cookie and (ref $cookie eq 'GT::CGI::Cookie'));
            push @headers, $cookie->cookie_header;
            $add_date = 1;
            $cookies++;
        }
    }
    elsif ($container) {
        $self->error ('INVALIDCOOKIE', 'WARN', $p{'-cookie'});
    }
    delete $p{'-cookie'};

# Print expiry if requested.
    if (defined $p{'-expires'}) {
        my $date = GT::CGI::Cookie::_format_date (' ', $p{'-expires'});
        if (! $date) {
            $self->error ('INVALIDDATE', 'WARN', $p{'-expires'});
        }
        else {
            push @headers, "Expires: $date";
            $add_date = 1;
        }
        delete $p{'-expires'};
    }

# Add a Date header if we printed an expires tag or a cookie tag.
    if ($add_date) {
        my $now = GT::CGI::Cookie::_format_date (' ');
        push @headers, "Date: $now";
    }

# Add Redirect Header.
    my $iis_redirect;
    if ($p{'-url'}) {
        if ($cookies and $ENV{SERVER_SOFTWARE} =~ /IIS/i) {
            $iis_redirect = $p{'-url'};
        }
        else {
            push @headers, "Location: $p{'-url'}";
        }
        delete $p{'-url'};
    }

# Add the Content-type header.
    if (@_ == 1 and !ref $_[0]) {
        push @headers, "Content-type: $_[0]";
    }
    elsif ( $p{ '-type' } ) {
        push @headers, "Content-type: $p{-type}";
        delete $p{'-type'};
    }
    else {
        push @headers, "Content-type: text/html";
    }

# Add any custom headers.
    foreach my $key ( keys %p ) {
        $key =~ m,^\s*-?(.+),;
        push @headers, escape("\u$1") . ": " . (ref $p{$key} eq 'SCALAR' ? ${$p{$key}} : escape($p{$key}));
    }
    $PRINTED_HEAD = 1;

    my $headers = join ($CRLF, @headers) . $CRLF . $CRLF;

# Fun hack for IIS
    if ($iis_redirect) {
        $iis_redirect =~ y/;/&/; # You can't have semicolons in a meta http-equiv tag.
        my $html = <<END_OF_HTML;
<html><head><title>Document Moved</title><meta http-equiv="refresh" content="0;URL=$iis_redirect"></head>
<body><noscript><h1>Object Moved</h1>This document may be found <a HREF="$iis_redirect">here</a></noscript></body></html>
END_OF_HTML
        return $headers . $html;
    }
    else {
        return $headers;
    }
}

sub redirect {
#--------------------------------------------------------------------------------
# Print a redirect header.
#
    my $self = shift;
    $self = $self->class_new unless ref $self;
    
    my (@headers, $url);
    if (@_ == 0) {
        return $self->header ( { -url => $self->self_url } );
    }
    elsif (@_ == 1) {
        return $self->header ( { -url => shift } );
    }
    else {
        my $opts = ref $_[0] eq 'HASH' ? shift : {@_};
        $opts->{'-url'} ||= $opts->{'-URL'} || $self->self_url;
        return $self->header ( $opts );
    }
}

sub unescape {
#--------------------------------------------------------------------------------
# returns the url decoded string of the passed argument
#
    my $todecode = (defined $_[0] and ((ref $_[0] eq 'GT::CGI') or ($_[0] eq 'GT::CGI'))) ? $_[1] : $_[0];
    return unless (defined $todecode);
    $todecode    =~ tr/+/ /;       # pluses become spaces
    $todecode    =~ s/%([0-9a-fA-F]{2})/chr(hex($1))/ge;
    return $todecode;
}

sub escape {
#--------------------------------------------------------------------------------
# return the url encoded string of the passed argument
#
    my $toencode = (defined $_[0] and ((ref $_[0] eq 'GT::CGI') or ($_[0] eq 'GT::CGI'))) ? $_[1] : $_[0];
    return unless (defined $toencode);
    $toencode    =~ s/([^\w. -])/sprintf("%%%02X",ord($1))/eg;
    $toencode    =~ s/ /%20/g;
    return $toencode;
}

sub html_escape {
#--------------------------------------------------------------------------------
# Return the string html_escaped.
#
    my $toencode = (defined $_[0] and ((ref $_[0] eq 'GT::CGI') or ($_[0] eq 'GT::CGI'))) ? $_[1] : $_[0];
    return unless (defined $toencode);
    if (ref($toencode) eq 'SCALAR') {
        $$toencode =~ s/&/&amp;/g;
        $$toencode =~ s/</&lt;/g;
        $$toencode =~ s/>/&gt;/g;
        $$toencode =~ s/"/&quot;/g;
    }
    else {
        $toencode =~ s/&/&amp;/g;
        $toencode =~ s/</&lt;/g;
        $toencode =~ s/>/&gt;/g;
        $toencode =~ s/"/&quot;/g;
    }
    return $toencode;
}

sub html_unescape {
#--------------------------------------------------------------------------------
# Return the string html unescaped.
#
    my $todecode = (defined $_[0] and ((ref $_[0] eq 'GT::CGI') or ($_[0] eq 'GT::CGI'))) ? $_[1] : $_[0];
    return unless (defined $todecode);
    if (ref ($todecode) eq 'SCALAR') {
        $$todecode =~ s/&lt;/</g;
        $$todecode =~ s/&gt;/>/g;
        $$todecode =~ s/&quot;/"/g;
        $$todecode =~ s/&amp;/&/g;
    }
    else {
        $todecode =~ s/&lt;/</g;
        $todecode =~ s/&gt;/>/g;
        $todecode =~ s/&quot;/"/g;
        $todecode =~ s/&amp;/&/g;
    }
    return $todecode;
}

sub self_url {
# -------------------------------------------------------------------
# Return full URL with query options as CGI.pm
#
    return $_[0]->url ( query_string => 1, absolute => 1 );
}

sub url {
# -------------------------------------------------------------------
# Return the current url. Can be called as GT::CGI->url() or $cgi->url().
#
    my $self = shift;
    $self = $self->class_new unless ref $self;
    $FORM_PARSED or $self->load_data();
    my $opts = $self->common_param(@_);

    my $absolute        = exists $opts->{absolute} ? $opts->{absolute} : 0;
    my $query_string    = exists $opts->{query_string} ? $opts->{query_string} : 1;
    my $path_info       = exists $opts->{path_info}    ? $opts->{path_info} : 0;
    if ($opts->{relative}) {
        $absolute = 0;
    }

    my $url = '';
    my $script = $ENV{SCRIPT_NAME} || $0;
    my ($path, $prog) = $script =~ m,^(.+?)[/\\]?([^/\\]*)$,;

    if ($absolute) {
        my ($protocol, $version) = split ('/', $ENV{SERVER_PROTOCOL} || 'HTTP/1.0');
        $url = lc $protocol . "://";

        my $host = $ENV{HTTP_HOST} || $ENV{SERVER_NAME} || '';
        $url .= $host;

        $path =~ s,^[/\\]*|[/\\]*$,,g;
        $url .= "/$path/";      
    }
    $prog =~ s,^[/\\]*|[/\\]*$,,g;
    $url .= $prog;
    
    if ($path_info and $ENV{PATH_INFO}) {
        if (defined $ENV{SERVER_SOFTWARE} && ($ENV{SERVER_SOFTWARE} =~ /IIS/)) {
            $ENV{PATH_INFO} =~ s,$ENV{SCRIPT_NAME},,;
        }
        $url .= $ENV{PATH_INFO};
    }
    if ($query_string) {
        my $qs = $self->query_string();
        if ($qs) {
            $url .= "?" . $qs;
        }
    }
    return $url;
}

sub query_string {
# -------------------------------------------------------------------
# Returns the query string url escaped.
#
    my $self = shift;
    $self = $self->class_new unless ref $self;
    $FORM_PARSED or $self->load_data();
    my $qs = '';
    foreach my $key (keys %{$self->{params}}) {
        my $esc_key = escape($key);
        foreach my $val (@{$self->{params}->{$key}}) {
            $qs .= $esc_key . "=" . escape($val) . ";";
        }
    }
    $qs and chop $qs;
    $qs ? return $qs : return '';
}

sub parse_str {
#--------------------------------------------------------------------------------
# parses a query string and add it to the HTTP_PARAMETER list
#
    my $self = shift;
    for ( split /[;&]/, shift ) {
        /([^=]+)=(.*)/ or next;
        my ($key, $val) = (unescape("$1"), unescape("$2"));

# Need to remove cr's on windows.
        if ($^O eq 'MSWin32') {
            $key =~ s/\r\n/\n/g;
            $val =~ s/\r\n/\n/g;
        }
        push @{$self->{params}->{$key}}, $val;
    }
}

package GT::CGI::Cookie;
#================================================================================
    use strict;
    use GT::Base;
    use vars   qw/@ISA $ATTRIBS/;

    @ISA     = qw/GT::Base/;

    $ATTRIBS = {
                '-name'     => '',
                '-value'    => '',
                '-expires'  => '',
                '-path'     => '',
                '-domain'   => '',
                '-secure'   => ''
            };
    my @MON  = qw/Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec/;
    my @WDAY = qw/Sun Mon Tue Wed Thu Fri Sat/;

sub cookie_header {
#--------------------------------------------------------------------------------
# Returns a cookie header.
#
    my $self    = shift;

# make sure we have a name to use..,
    $self->{'-name'} or return;

    my $name    = GT::CGI::escape($self->{'-name'});
    my $value   = GT::CGI::escape($self->{'-value'});

# build the header that creats the cookie
    my $header  = "Set-Cookie: $name=$value";

    $self->{'-expires'} and $header .= "; expires=" . _format_date ('-', $self->{'-expires'});
    $self->{'-path'}    and $header .= "; path=$self->{-path}";
    $self->{'-domain'}  and $header .= "; domain=$self->{-domain}";
    $self->{'-secure'}  and $header .= "; secure";

    return "$header";
}

sub _format_date {
# -------------------------------------------------------------------
# Return a string in http_gmt format, but accepts one in unknown format.
#   Wed, 23 Aug 2000 21:20:14 GMT
#
    my ($sep, $datestr) = @_;
    my $unix_time = defined $datestr ? _expire_calc($datestr) : time();

    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = gmtime ($unix_time);
    $year = $year + 1900;

    my $date = sprintf("%s, %02d$sep%s$sep%04d %02d:%02d:%02d GMT",
                    $WDAY[$wday],$mday,$MON[$mon],$year,$hour,$min,$sec);
    return $date;
}

sub _expire_calc {
# -------------------------------------------------------------------
# Calculates when a date based on +- times. See CGI.pm for more info.
#
    my $time = shift;
    my %mult = qw/s 1 m 60 h 3600 d 86400 M 2592000 y 31536000/;
    my $offset;

    if (! $time or (lc $time eq 'now')) {
        $offset = 0;
    }
    elsif ($time =~ /^\d+/) {
        return $time;
    }
    elsif ($time=~/^([+-]?(?:\d+|\d*\.\d*))([smhdMy]?)/) {
        $offset = ($mult{$2} || 1)*$1;
    }
    else {
        return $time;
    }
    return time() + $offset;
}

package GT::CGI::MultiPart;
__DATA__
# ================================================================================
# This is taken almost entirely from CGI.pm, and is loaded on demand.
#
    use strict;
    no strict 'refs';
    use GT::Base;
    use GT::TempFile ();
    use vars    qw/$DEBUG $BLOCK_SIZE $CRLF $ERRORS @ISA $MAX_READS $ATTRIBS/;

    @ISA           = ('GT::Base');
    $BLOCK_SIZE    = 4096;    
    $MAX_READS     = 2000;
    $ATTRIBS       = {
                        fh       => undef,      # web request on stdin
                        buffer   => '',         # buffer to hold tmp data
                        length   => 0,          # length of file to parse
                        boundary => undef,      # mime boundary to look for
                        fillunit => $BLOCK_SIZE,# amount to read per chunk
                        safety   => 0           # safety counter
                    };
    $ERRORS        = {
                        'NOBOUNDARY' => "Unable to find a MIME boundary in environment. Content-type looks like: %s",
                        'CLIENTABORT' => "Unable to read data from server. Still have %s bytes to read, but got 0. Data in buffer is: %s",
                        'BADMULTIPART' => "Invalid multipart message. Nothing left to read, and can't find closing boundary. Data in buffer is: %s",
                    };

sub parse {
# -------------------------------------------------------------------
# Parses a multipart form to handle file uploads.
#
    my ($class, $cgi) = @_;

# We override any fatal handlers as our handlers typically create a CGI object
# avoiding a nasty loop.
    local $SIG{__DIE__} = sub { die @_; };
   
# We only load the multipart parser if we have multipart code.
    my $parser = $class->new or return;

    my ($header, $name, $value, $filename);
    while (!$parser->eof) {
        $header                = $parser->read_header or return die "BADREQUEST";
        ($name, my $temp_name) = $header->{'Content-Disposition'} =~ m/ name=(?:"([^"]*)"|((?!")[^;]*))/;
        $name = $temp_name if $temp_name and not $name;
        ($filename) = $header->{'Content-Disposition'} =~ m/ filename="?([^\";]*)"?/;

        defined $cgi->{params}->{$name} or ($cgi->{params}->{$name} = []);

# Not a file, just regular form data.
        if (! defined $filename or $filename eq '') {
            $value = $parser->read_body;

# Netscape 6 does some fun things with line feeds in 
# multipart form data
            $value =~ s/\r\r/\r/g;      # What it does on unix
            if ($^O eq 'MSWin32') {
                $value =~ s/\r\n/\n/g;
            }
            push @{$cgi->{params}->{$name}}, $value;
            next;
        }

# Print out the data to a temp file.
        local $\ = '';
        my $tmp_file = new GT::TempFile;
        my $fh       = new GT::CGI::Fh ( $filename, $$tmp_file, 0);
        binmode ($fh);
        my $data     = '';
        while (defined ($data = $parser->read)) {
            print $fh $data;
        }
        seek ($fh, 0, 0);
        binmode ($fh);
        push @{$cgi->{params}->{$name}}, $fh;
    }
}

sub init {
# -------------------------------------------------------------------
# Initilize our object.
#
    $DEBUG   = $GT::CGI::DEBUG;
    $CRLF    = $GT::CGI::CRLF;

    my $self = shift;

# Get the boundary marker.
    my $boundary;
    if (defined $ENV{CONTENT_TYPE} and ($ENV{CONTENT_TYPE} =~ /boundary=\"?([^\";,]+)\"?/)) {
        $boundary  = $1;
    }
    else {
        return $self->error ("NOBOUNDARY", "FATAL", $ENV{CONTENT_TYPE});
    }
    $self->{boundary} = "--$boundary";

# Get our filehandle.
    binmode(main::STDIN);
    
# And if the boundary is > the BLOCK_SIZE, adjust.
    if (length $boundary > $self->{fillunit}) {
        $self->{fillunit} = length $boundary;
    }

# Set the content-length.
    $self->{length} = defined $ENV{CONTENT_LENGTH} ? $ENV{CONTENT_LENGTH} : 0;

# Read the preamble and the topmost (boundary) line plus the CRLF.
    while ($self->read) { }
}

sub fill_buffer {
# -------------------------------------------------------------------
# Fill buffer.
#
    my ($self, $bytes) = @_;

    return unless $self->{length};

    my $boundary_length = length $self->{boundary};
    my $buffer_length   = length $self->{buffer};
    my $bytes_to_read   = $bytes - $buffer_length + $boundary_length + 2;
    $bytes_to_read      = $self->{length} if ($self->{length} < $bytes_to_read);

    my $bytes_read = read (main::STDIN, $self->{buffer}, $bytes_to_read, $buffer_length);
    if (! defined $self->{buffer}) {
        $self->{buffer} = '';
    }
    if ($bytes_read == 0) {
        if ($self->{safety}++ > $MAX_READS) {
            return $self->error ('CLIENTABORT', 'FATAL', $self->{length}, $self->{buffer});
        }
    }
    else {
        $self->{safety} = 0;
    }

    $self->{length} -= $bytes_read;
}

sub read {
# -------------------------------------------------------------------
# Read some input.
#
    my $self  = shift;
    my $bytes = $self->{fillunit};

# Load up self->{buffer} with data.
    $self->fill_buffer ($bytes);

# find the boundary (if exists).
    my $start = index ($self->{buffer}, $self->{boundary});

# Make sure the post was formed properly.
    unless (($start >=0) or ($self->{length} > 0)) {
        return $self->error ('BADMULTIPART', 'FATAL', $self->{buffer});
    }

    if ($start == 0) {
# Quit if we found the last boundary at the beginning.
        if (index ($self->{buffer},"$self->{boundary}--")==0) {
            $self->{buffer} = '';
            $self->{length} = 0;
            return;
        }
# Otherwise remove the boundary (+2 to remove line feeds).
        substr ($self->{buffer}, 0, length ($self->{boundary}) + 2) = '';
        return;
    }
    
    my $bytes_to_return;
    if ($start > 0) {
        $bytes_to_return = $start > $bytes ? $bytes : $start;
    }
    else {
        $bytes_to_return = $bytes - (length ($self->{boundary}) + 1);
    }

    my $return = substr ($self->{buffer}, 0, $bytes_to_return);
    substr ($self->{buffer}, 0, $bytes_to_return) = '';

    return ($start > 0) ? substr ($return, 0, -2) : $return;
}

sub read_header {
# -------------------------------------------------------------------
# Reads the header.
#
    my $self = shift;
    my ($end, $ok, $bad, $safety);
    $ok = $bad = 0;
    do {
        $self->fill_buffer($self->{fillunit});

        $ok++ if (($end = index ($self->{buffer}, "$CRLF$CRLF")) >= 0);
        $ok++ if ($self->{buffer} eq '');
        $bad++ if (!$ok and $self->{length} <= 0);
        ($safety++ > 10) and return;
    } until ($ok || $bad);
    
    return if ($bad);

    my $header = substr ($self->{buffer}, 0, $end + 2);
    substr ($self->{buffer}, 0, $end + 4) = '';

    my %header;
    my $token = '[-\w!\#$%&\'*+.^_\`|{}~]';
    $header   =~ s/$CRLF\s+/ /og;
    while ($header =~ /($token+):\s+([^$CRLF]*)/go) {
        my ($field_name,$field_value) = ($1,$2);
        $field_name =~ s/\b(\w)/\u$1/g; 
        $header{$field_name} = $field_value;
    }
    return \%header;
}

sub read_body {
# -------------------------------------------------------------------
# Reads a body and returns as a single scalar value.
#
    my $self   = shift;
    my $data   = '';
    my $return = '';
    while (defined ($data = $self->read)) {
        $return .= $data;
    }
    return $return;
}

sub eof {
# -------------------------------------------------------------------
# Return true when we've finished reading.
#
    my $self = shift;
    return 1 if ((length $self->{buffer} ==0) and ($self->{length} <= 0));
}

package GT::CGI::Fh;
# ===================================================================
# Magic File Handle, let's you print the file name, but also act like
# a file handle for read, just like CGI.pm.
#
    use strict;
    no  strict 'refs';
    use vars   qw/$FH/;
    use overload 
        '""'  => \&as_string,
        'cmp' => \&compare,
        'fallback' => 1;
    $FH = 1;

sub new {
# -------------------------------------------------------------------
# Create a new filehandle based on a counter, and the filename.
#
    my ($pkg, $name, $file, $delete) = @_;
    require Fcntl unless defined &Fcntl::O_RDWR;
    my $fname = sprintf ("FH%05d%s", $FH++, $name);
    
    $fname =~ s/([:'%])/ sprintf '%%%02X', ord $1 /eg;
    my $fh = \do { local *{$fname}; *{$fname} };

    sysopen ($fh, $file, Fcntl::O_RDWR()|Fcntl::O_EXCL(),0600) || die "Can't open file: $file ($!)";
    unlink  ($file) if $delete;
    bless $fh, $pkg;

    return $fh;
}

sub as_string {
# -------------------------------------------------------------------
# Return the filename, strip off leading junk first.
#
    my $self = shift;
    my $fn   = $$self;
    $fn =~ s/%(..)/ chr(hex($1)) /eg;
    $fn =~ s/^\*GT::CGI::Fh::FH\d{5}//;
    return $fn;
}

sub compare {
# -------------------------------------------------------------------
# Do comparisions, uses as_string to get file name first.
#
    my $self  = shift;
    my $value = shift;
    return "$self" cmp $value;
}

sub DESTROY {
# -------------------------------------------------------------------
# Close file handle.
#
    my $self = shift;
    close $self;
}

1;

=head1 NAME

GT::CGI - a lightweight replacement for CGI.pm

=head1 SYNOPSIS

    use GT::CGI;
    my $in = new GT::CGI;
    foreach my $param ($in->param) {
        print "VALUE: $param => ", $in->param($param), "\n";
    }

    use GT::CGI qw/-no_parse_buttons/;

=head1 DESCRIPTION

GT::CGI is a lightweight replacement for CGI.pm. It implements most
of the functionality of CGI.pm, with the main difference being no 
function based interface, and no html functionality.

The motivation for this is to provide a CGI module that can be shipped
with Gossamer products, not having to depend on a current version
of CGI.pm being installed on remote servers.

Credit and thanks goes to the author of CGI.pm. A lot of the code
(especially file upload) was taken from CGI.pm.

=head2 param - Accessing form input.

Can be called as either a class method or object method. When
called with no arguments a list of keys is returned.

When called with a single argument in scalar context the first
(and possibly only) value is returned. When called in list context
an array of values is returned. 

When called with two arguments, it sets the key value pair.

=head2 header() - Printing HTTP headers

Can be called as a class method or object method. When called 
with no arguments, simply returns the HTTP header.

Other options include:

=over 4

=item -force => 1

Force printing of header even if it has already been displayed.

=item -type => 'text/plain'

Set the type of the header to something other then text/html.

=item -cookie => $cookie

Display any cookies. You can pass in a single GT::CGI::Cookie 
object, or an array of them.

=item -nph => 1

Display full headers for nph scripts.

=back

If called with a single argument, sets the Content-Type.

=head2 redirect - Redirecting to new URL.

Returns a Location: header to redirect a user. 

=head2 cookie - Set/Get HTTP Cookies.

Sets or gets a cookie. To retrieve a cookie:

    my $cookie = $cgi->cookie ('key');
    my $cookie = $cgi->cookie (-name => 'key');

or to retrieve a hash of all cookies:

    my $cookies = $cgi->cookie;

To set a cookie:

    $c = $cgi->cookie (-name => 'foo', -value => 'bar')

You can also specify -expires for when the cookie should expire,
-path for which path the cookie valid, -domain for which domain
the cookie is valid, and -secure if the cookie is only valid for
secure sites.

You would then set the cookie by passing it to the header function:

    print $in->header ( -cookie => $c );

=head2 url - Retrieve the current URL.

Returns the current URL of the script. It defaults to display
just the script name and query string.

Options include:

=over 4

=item absolute => 1

Return the full URL: http://domain/path/to/script.cgi

=item relative => 1

Return only the script name: script.cgi

=item query_string => 1

Return the query string as well: script.cgi?a=b

=item path_info => 1

Returns the path info as well: script.cgi/foobar

=back 4

=head2 get_hash - Return all form input as hash.

This returns the current parameters as a hash. Any values that have
the same key or joined as a string with $INPUT_SEPARATOR. get_hash
takes an optional argument that determines whether input is joined on
input separator (if true or not specified) or only the first argument
is returned (if false).

=head2 escape - URL escape a string.

Returns the passed in value URL escaped. Can be called as
class method or object method.

=head2 unescape - URL unescape a string.

Returns the passed in value URL un-escaped. Can be called as
class method or object method.

=head2 html_escape - HTML escape a string

Returns the passed in value HTML escaped. Translates &, <, >
and " to their html equivalants.

=head2 html_unescape - HTML unescapes a string

Returns the passed in value HTML unescaped.

=head1 DEPENDANCIES

Note: L<GT::CGI> depends on GT::Base, and if you are using file uploads,
L<GT::TempFile>.

=head1 COPYRIGHT

Copyright (c) 2000 Gossamer Threads Inc.  All Rights Reserved.
http://www.gossamer-threads.com/

=head1 VERSION

Revision: $Id: CGI.pm,v 1.93 2001/11/28 03:16:54 jagerman Exp $

=cut
