# ==================================================================
# MySQLMan - web based MySQL manager.
#
#   Website  : http://gossamer-threads.com/
#   Support  : http://gossamer-threads.com/scripts/support/
#   Revision : $Id: html.pl,v 1.20 2002/01/03 01:46:17 alex Exp $
# 
# Copyright (c) 2000 Gossamer Threads Inc.  All Rights Reserved.
# Redistribution in part or in whole strictly prohibited. 
# ==================================================================
#
# Html.pl uses both GT::Base and GT::Template to substitude variables
# in the templates with real values.  Variables in the templates are in
# the form <%variable_name%>.  For more detailed usage of GT::Template
# please refer to the pod in GT::Template.
#
# Common Variables
# --------------------------------------------------------
# home_url   : the URL to home page.  (Specified in mysql.cfg)
# script_url : the URL to mysql.cgi.  (Specified in mysql.cfg)
# db         : the name of current database.
# table      : the name of current table.
# version    : the current version of the script
# help_topic : help_topic for sub html_help
#

sub html_database{
# --------------------------------------------------------
# List of databases in MySQL.
#     db_table : the table that contain the databases in MySQL 
#                together with "drop" links to drop databases.
#     $feedback: feedback message of any action performed before 
#                arriving the page.

    my ($in, $data_source, $db_table, $feedback) = @_;
    GT::Template->parse ('database.html', {
                                    do          => scalar $in->param('do'),
                                    home_url    => $CONFIG{'home_url'}, 
                                    script_url  => $CONFIG{'script_url'}, 
                                    db_table    => $db_table,
                                    data_source => scalar $in->html_escape($data_source),
                                    table       => '',
                                    feedback    => scalar $in->html_escape($feedback),
                                    help_topic  => 'db_list',
                                    version     => $VERSION,
                                    },
                                    {
                                        root => $CONFIG{template_dir},
                                        print => 1
                                    });
}

sub html_table{
# --------------------------------------------------------
# List of tables in the database selected.
#     table_tables: the table that contains the table names in 
#                   the database together with action links
#                   (browse/select/property/insert/empty/drop)
#     $feedback   : feedback message of any action performed before 
#                   arriving the page.   

    my ($in, $table_tables, $feedback) = @_;
    GT::Template->parse ('table.html', { 
                                    do              => scalar $in->param('do'),
                                    home_url        => $CONFIG{'home_url'},
                                    script_url      => $CONFIG{'script_url'}, 
                                    data_source     => scalar $in->param('data_source'),
                                    db              => get_db($in->param('data_source')),
                                    table_tables    => $table_tables,
                                    table           => '',
                                    from_monitor    => scalar $in->param('from_monitor'),
                                    feedback        => scalar $in->html_escape($feedback),
                                    help_topic      => "table_list",
                                    version         => $VERSION,
                                    },
                                    {
                                        root => $CONFIG{template_dir},
                                        print => 1
                                    });
}

sub html_table_browse{
# --------------------------------------------------------
# This will display the result of any query that requires records to 
# be displayed.
#     page         : the current page.
#     page_jump    : the input box that allows the user to go to any page.
#     page_link    : the links to prev/next/top page.
#     col_name     : the column names in the table.
#     table_records: the table that contains the result of the query.
#     query_printed: the query executed.
#     total_rows   : total number of rows resulted from the query.
#     
    my ($in, $table, $page_jump, $page_link, $col_name, $table_records, $query, $empty_set, 
        $pri_key, $query_printed, $total_rec_num, $save_search_link) = @_;

	my $query_table = build_saved_query_table($in);

    GT::Template->parse ('table_browse.html', {
                                    total_rows      => $total_rec_num,
                                    do              => scalar $in->param('do'),
                                    home_url        => $CONFIG{'home_url'},
                                    script_url      => $CONFIG{'script_url'}, 
                                    data_source     => scalar $in->param('data_source'),
                                    db              => &get_db($in->param('data_source')),
                                    table           => $table,
                                    page            => scalar $in->param('page') || '1',
                                    empty_set       => $empty_set,
                                    page_jump       => $page_jump,
                                    page_link       => $page_link,
                                    col_name        => $col_name,
                                    table_records   => $table_records,
                                    query           => scalar $in->html_escape($query),
                                    query_printed   => $query_printed || $query, 
                                    action          => scalar $in->param('browse_action') || $in->param('action'),
                                    pri_key         => $pri_key,
                                    from_monitor    => scalar $in->param('from_monitor'),
                                    help_topic      => "browse",
                                    version         => $VERSION,
									query_table     => $query_table,
									save_search_link => $save_search_link
                                    },
                                    {
                                        root => $CONFIG{template_dir},
                                        print => 1
                                    });
}

sub html_table_select{
# --------------------------------------------------------
# Search page that allows the user to compose a SELECT query.
#     select_table : the fields that are available for the SELECT query.
#     example_table: table input fields that lets the user to construct a 
#                    query to "query by example."

    my ($in, $select_table, $example_table) = @_;
    GT::Template->parse ('table_select.html', { 
                                    do              => scalar $in->param('do'),
                                    home_url        => $CONFIG{'home_url'},
                                    script_url      => $CONFIG{'script_url'}, 
                                    data_source     => scalar $in->param('data_source'),
                                    db              => &get_db($in->param('data_source')),
                                    table           => scalar $in->param('table'),
                                    select_table    => $select_table,
                                    example_table   => $example_table,
                                    help_topic      => "select",
                                    version         => $VERSION,
                                    },
                                    {
                                        root => $CONFIG{template_dir},
                                        print => 1
                                    });
}

sub html_property{
# --------------------------------------------------------
# Displays the specificaions of a table. 
#     $table_property: the table that consists of the table spec's
#                      and action links (change/drop/primary/index/unique).
#     key_table      : the table that shows the keys in the table.
#     $feedback      : feedback message of any action performed before 
#                      arriving the page.
     

    my ($in, $table_property, $feedback) = @_;
    GT::Template->parse ('property.html', { 
                                    do              => scalar $in->param('do'),
                                    home_url        => $CONFIG{'home_url'},
                                    script_url      => $CONFIG{'script_url'}, 
                                    data_source     => scalar $in->param('data_source'),
                                    db              => &get_db($in->param('data_source')) ||'',
                                    table           => scalar $in->param('table'),
                                    table_property  => $table_property,
                                    key_table       => &get_key_table($in),
                                    feedback        => scalar $in->html_escape($feedback),
                                    help_topic      => "properties",
                                    version         => $VERSION,
                                    },
                                    {
                                        root => $CONFIG{template_dir},
                                        print => 1
                                    });
}

sub html_insert{
# --------------------------------------------------------
# Insert new record.
#     $form_fields: the input table that lets the user to enter
#                   values in to create a new record.
#     $feedback   : feedback message of any action performed before 
#                   arriving the page.

    my ($in, $feedback) = @_;

    my $table = $in->param('table');
    if ($CONFIG{'demo_mode'}) { &alias_name_check($table, $in) or return; }
    my $form_fields = &form_fields($in, 0, ()) or return;

    GT::Template->parse ('insert.html', { 
                                    do              => scalar $in->param('do'),
                                    home_url        => $CONFIG{'home_url'},
                                    script_url      => $CONFIG{'script_url'}, 
                                    data_source     => scalar $in->param('data_source'),
                                    db              => &get_db($in->param('data_source'))|| '',
                                    table           => scalar $in->param('table'),
                                    insert_fields   => $form_fields,
                                    feedback        => scalar $in->html_escape($feedback),
                                    help_topic      => "insert",
                                    version         => $VERSION,
                                    },
                                    {
                                        root => $CONFIG{template_dir},
                                        print => 1
                                    });
}

sub html_update{
# --------------------------------------------------------
# Edit record page.
#     $insert_fields: the input table with original values of the record 
#                     in the input fields.

    my ($in, $insert_fields) = @_;
    GT::Template->parse ('edit.html', { 
                                    do              => scalar $in->param('do'),
                                    home_url        => $CONFIG{'home_url'},
                                    script_url      => $CONFIG{'script_url'}, 
                                    data_source     => scalar $in->param('data_source'),
                                    db              => &get_db($in->param('data_source')),
                                    table           => scalar $in->param('table'),
                                    insert_fields   => $insert_fields,
                                    record_modify   => scalar $in->html_escape($in->param('record_modify')),
                                    page            => scalar $in->param('page') || 1,
                                    action          => scalar $in->param('action'),
                                    sort_index      => scalar $in->html_escape($in->param('sort_index')),
                                    fields          => scalar $in->html_escape($in->param('fields')),
                                    where           => scalar $in->html_escape($in->param('where')), 
                                    example         => scalar $in->html_escape($in->param('example')),
                                    browse_action   => scalar $in->param('browse_action'),
                                    help_topic      => 'edit',
                                    version         => $VERSION,
                                    },
                                    {
                                        root => $CONFIG{template_dir},
                                        print => 1
                                    });
}

sub html_table_def{
# --------------------------------------------------------
# lets the user to construct the specificaitons of fields/columns. 
# Used when creating a new table or adding new field(s)/column(s).
# If a new table is to be created, the table name is first checked to 
# see if it is a valid one.  Then the number of fields/columns is checked
# also for its validity.
#     $columns: the input table that lets user to construct field/column
#               spec's.        
#

    my ($in, $action) = @_;
    my ($columns);

    if ($action eq 'create') { $do = 'create_table'; }
    else { $do = 'alter_table'; }

    &valid_name_check($in->param('table')) or return;

    if ($in->param('num_of_fields') < 1)    { &sqlerr("Number of fields cannot be less than 0.") }
    elsif ($in->param('num_of_fields')>500) {&sqlerr("Number of fields too large.") }    
    else {
        for (my $i = 0; $i < $in->param('num_of_fields'); $i++) {
            $columns .= GT::Template->parse ('create_field.txt', { field_num => $i }, { root => $CONFIG{template_dir} });    
        }

        GT::Template->parse ('create_table.html', { 
                                    do              => $do,
                                    home_url        => $CONFIG{'home_url'},
                                    script_url      => $CONFIG{'script_url'}, 
                                    data_source     => scalar $in->param('data_source'),
                                    db              => &get_db($in->param('data_source')),
                                    table           => scalar $in->param('table'),
                                    columns         => $columns,
                                    num_of_fields   => scalar $in->param('num_of_fields'),
                                    position        => scalar $in->param('position'),
                                    help_topic      => 'col_def',
                                    version         => $VERSION,
                                    },
                                    {
                                        root => $CONFIG{template_dir},
                                        print => 1
                                    });
    }
}

sub html_alter_col{
# --------------------------------------------------------
# Change field/column specification.
#     col    : the field/column name that's being changed.
#     field  : same as col.
#     type   : the original field/column type.
#     null   : either 'YES' or ''.
#     default: the default value.
#     extra  : extra properties. (auto_increment)

    my ($in, $field, $type, $length_set, $attributes, $null, $default, $extra) = @_;
    my ($type_select, $null_select, $extra_select);

# show the original field/column specs. 
    $type_select = "<option>$type</option>";

    if ($attributes) { $attribute_select = "<option>$attributes</option>"}

    if ($null) { $null_select = '<option>null</option>' }
    else { $null_select = '<option>not null</option>' }

    if ($extra) {$extra_select = "<option>$extra</option>"} 

    GT::Template->parse ('alter_col.html', { 
                                    do              => scalar $in->param('do'),
                                    home_url        => $CONFIG{'home_url'},
                                    script_url      => $CONFIG{'script_url'}, 
                                    data_source     => scalar $in->param('data_source'),
                                    db              => &get_db($in->param('data_source')),
                                    table           => scalar $in->param('table'),
                                    col             => scalar $in->param('col'),
                                    field           => $field, 
                                    type            => $type,
                                    length_set      => scalar $in->html_escape($length_set),
                                    attributes      => $attributes,
                                    null            => $null, 
                                    default         => $default, 
                                    extra           => $extra,
                                    type_select     => $type_select,
                                    null_select     => $null_select,
                                    extra_select    => $extra_select,
                                   attribute_select => $attribute_select,
                                   help_topic       => 'col_def_change',
                                   version          => $VERSION,
                                    },
                                    {
                                        root => $CONFIG{template_dir},
                                        print => 1
                                    });
}

sub html_page_jump{
# --------------------------------------------------------
# displays a text input box that allow user to go to 
# any page of the result table.  
#     pages: total number of pages of the result.

    my ($in, $pages, $fields, $example, $index) = @_;
    GT::Template->parse ('page_jump.txt', {
                                    do              => scalar $in->param('do'),
                                    script_url      => $CONFIG{'script_url'}, 
                                    data_source     => scalar $in->param('data_source'),
                                    table           => scalar $in->param('table'),
                                    pages           => $pages,
                                    fields          => $fields,
                                    where           => scalar $in->param('where'),
                                    action          => scalar $in->param('action'),
                                    sort_index      => $index,
                                    example         => scalar $in->html_escape($example),
                                    query           => scalar $in->html_escape($in->param('query')),
                                    },
                                    {
                                        root => $CONFIG{template_dir},
                                        print => 1
                                    });
}

sub html_confirm_action{
# --------------------------------------------------------
# Confirmation page.  
#     $query: the actual query that is going to be executed.

    my ($in, $query) = @_;

    GT::Template->parse ('confirm.html', { 
                                    home_url        => $CONFIG{'home_url'},
                                    script_url      => $CONFIG{'script_url'},
                                    do              => scalar $in->param('do'),
                                    data_source     => scalar $in->param('data_source'),
                                    db              => &get_db($in->param('data_source')),
                                    table           => scalar $in->param('table'),
                                    action          => scalar $in->param('action'),
                                    query           => $query,
                                    query_param     => scalar $in->html_escape($query),
                                    col             => scalar $in->param('col'),
                                    key_name        => scalar $in->param('key_name'),
                                    redirect_url    => scalar $in->html_escape( $in->cookie($CONFIG{'url_cookie_name'}) ) || $CONFIG{'script_url'},
                                    help_topic      => 'confirm',
                                    version         => $VERSION,
                                    },
                                    {
                                        root => $CONFIG{template_dir},
                                        print => 1
                                    });
}


sub html_sqlerr{
# --------------------------------------------------------
# Error message.
#     error: the actual error message.

    my ($in, $error) = @_;
    my $table = $in->param('table');
    if (($in->param('do') eq 'create') || $in->param('action') eq 'rename_table') { $table = ''; }

    GT::Template->parse ('sqlerr.html', { 
                                    do              => scalar $in->param('do'),
                                    home_url        => $CONFIG{'home_url'},
                                    script_url      => $CONFIG{'script_url'},
                                    data_source     => scalar $in->param('data_source'),
                                    db              => &get_db($in->param('data_source')),
                                    table           => $table,
                                    error           => $error,
                                    help_topic      => 'sqlerr',
                                    version         => $VERSION,
                                    },
                                    {
                                        root => $CONFIG{template_dir},
                                        print => 1
                                    });
}

sub html_login{
# --------------------------------------------------------
# Login page.  The URL of the previous page is read in from cookie 
# or the script URL is used if the cookie is not set.  
#     url    : the URL to the previous page.
#     user   : Username.  Read from cookie.
#     pass   : Password.  Read from cookie.
#     host   : Host name.  Used for database connection and is read from cookie.
#     message: message displayed in the login page.

    my ($in, $message, $init_login) = @_;
    my $url = $in->cookie($CONFIG{'url_cookie_name'}) || $CONFIG{'script_url'};

    # For Netscape.
    $url =~ s/\;/&/g;

    if ($url =~ m/\?/) { $url .= '&from_login=1'}
    else { $url .= '?from_login=1' }

    GT::Template->parse ('login.html', {
                                    do              => scalar $in->param('do'),
                                    home_url        => $CONFIG{'home_url'},
                                    script_url      => $CONFIG{'script_url'},
                                    data_source     => scalar $in->param('data_source'),
                                    db              => &get_db($in->param('data_source')),
                                    table           => scalar $in->param('table'),
                                    url             => $url,
                                    user            => scalar $in->cookie($CONFIG{'db_user_cookie_name'}),
                                    pass            => scalar $in->cookie($CONFIG{'db_pass_cookie_name'}),
                                    host            => scalar $in->cookie($CONFIG{'db_host_cookie_name'}),
                                    message         => $message,
                                    init_login      => $init_login,
                                    help_topic      => 'login',
                                    version         => $VERSION,
                                    },
                                    {
                                        root => $CONFIG{template_dir},
                                        print => 1
                                    });
}


sub html_login_dbname{
# --------------------------------------------------------
# Login page.  The URL of the previous page is read in from cookie 
# or the script URL is used if the cookie is not set.  
#     url    : the URL to the previous page.
#     user   : Username.  Read from cookie.
#     pass   : Password.  Read from cookie.
#     host   : Host name.  Used for database connection and is read from cookie.
#     message: message displayed in the login page.

    my ($in, $message, $init_login) = @_;
    my $url = $in->cookie($CONFIG{'url_cookie_name'}) || $CONFIG{'script_url'};
    
    GT::Template->parse ('login_dbname.html', {
                                    do              => scalar $in->param('do'),
                                    home_url        => $CONFIG{'home_url'},
                                    script_url      => $CONFIG{'script_url'},
                                    data_source     => scalar $in->param('data_source'),
                                    db              => &get_db($in->param('data_source')),
                                    table           => scalar $in->param('table'),
                                    url             => $url,
                                    message         => scalar $in->html_escape($message),
                                    init_login      => $init_login,
                                    help_topic      => 'login_dbname',
                                    version         => $VERSION,
                                    },
                                    {
                                        root => $CONFIG{template_dir},
                                        print => 1
                                    });
}


sub html_logout{
# --------------------------------------------------------
# Logout message.

    GT::Template->parse ('logout.html', {
                                    home_url        => $CONFIG{'home_url'},
                                    script_url      => $CONFIG{'script_url'},
                                    data_source     => '',
                                    table           => '',
                                    help_topic      => 'logout',
                                    version         => $VERSION,
                                    },
                                    {
                                        root => $CONFIG{template_dir},
                                        print => 1
                                    });
}

sub html_back{
# --------------------------------------------------------
# The confirmation page that is displayed after login.
#     url: the URL to the page before the login command is performed.

    my $in = shift;
    GT::Template->parse ('login_back.html', {
                                    do              => scalar $in->param('do'),
                                    home_url        => $CONFIG{'home_url'},
                                    script_url      => $CONFIG{'script_url'},
                                    data_source     => scalar $in->param('data_source'),
                                    db              => &get_db($in->param('data_source')),
                                    table           => scalar $in->param('table'),
                                    url             => scalar $in->param('url') || "$CONFIG{'script_url'}",
                                    help_topic      => 'login_back',
                                    version         => $VERSION,
                                    },
                                    {
                                        root => $CONFIG{template_dir},
                                        print => 1
                                    });
}

#=================================================#
#             Top Level Operations                #
#=================================================#

sub html_create_db{
# --------------------------------------------------------
# Create new database.

    my $in = shift;
    GT::Template->parse ('op_create_db.html', {
                                    do              => scalar $in->param('do'),
                                    home_url        => $CONFIG{'home_url'},
                                    script_url      => $CONFIG{'script_url'},
                                    data_source     => scalar $in->param('data_source'),
                                    db              => &get_db($in->param('data_source')),
                                    table           => scalar $in->param('table'),
                                    help_topic      => 'create_db',
                                    version         => $VERSION,
                                    },
                                    {
                                        root => $CONFIG{template_dir},
                                        print => 1
                                    });
}
sub html_sql_monitor{
# --------------------------------------------------------
# SQL monitor.
#     $message: the feedback message from SQL monitor 
#               in the form "num_rows rows affacted"

    my ($in, $message) = @_;

# build a table for queries saved.
	my $query_table = build_saved_query_table($in);
 
	GT::Template->parse ('op_sql_monitor.html', {
                                    do              => scalar $in->param('do'),
                                    home_url        => $CONFIG{'home_url'},
                                    script_url      => $CONFIG{'script_url'},
                                    data_source     => scalar $in->param('data_source'),
                                    db              => &get_db($in->param('data_source')),
                                    table           => scalar $in->param('table'),
                                    feedback        => $message,
                                    help_topic      => 'sql_monitor',
                                    version         => $VERSION,
									query_table		=> $query_table
                                    },
                                    {
                                        root => $CONFIG{template_dir},
                                        print => 1
                                    });
}

sub html_create_table{
# --------------------------------------------------------
# Create new table page.

    my $in = shift;
    GT::Template->parse ('op_create_table.html', {
                                    do              => scalar $in->param('do'),
                                    home_url        => $CONFIG{'home_url'},
                                    script_url      => $CONFIG{'script_url'},
                                    data_source     => scalar $in->param('data_source'),
                                    table           => '',
                                    db              => &get_db($in->param('data_source')),
                                    help_topic      => 'create_table',
                                    version         => $VERSION,
                                    },
                                    {
                                        root => $CONFIG{template_dir},
                                        print => 1
                                    });
}
sub html_import{
# --------------------------------------------------------
# Data import page.

    my ($in, $options, $size) = @_;
    my $empty_op = '';
    
    for (my $i; $i<$size; $i++) {
        $empty .= '<OPTION value=""></OPTION>' . "\n";
    }
    
    $size = $size + 1;
    if ($size > 10) {$size = 10;}

    GT::Template->parse ('op_import.html', {
                                    do              => scalar $in->param('do'),
                                    home_url        => $CONFIG{'home_url'},
                                    script_url      => $CONFIG{'script_url'},
                                    data_source     => scalar $in->param('data_source'),
                                    db              => &get_db($in->param('data_source')),
                                    table           => scalar $in->param('table'),
                                    field_options   => $options,
                                    empty_options   => $empty,
                                    size            => $size,
                                    help_topic      => 'import',
                                    version         => $VERSION,
                                    },
                                    {
                                        root => $CONFIG{template_dir},
                                        print => 1
                                    });

}
sub html_export{
# --------------------------------------------------------
# Data export page.

    my ($in, $options, $size) = @_;
    my $empty_op = '';
    
    for (my $i; $i<$size; $i++) {
        $empty .= '<OPTION value=""></OPTION>' . "\n";
    }
    
    $size = $size + 1;
    if ($size > 10) {$size = 10;}
    GT::Template->parse ('op_export.html', {
                                    do              => scalar $in->param('do'),
                                    home_url        => $CONFIG{'home_url'},
                                    script_url      => $CONFIG{'script_url'},
                                    data_source     => scalar $in->param('data_source'),
                                    db              => &get_db($in->param('data_source')),
                                    table           => scalar $in->param('table'),
                                    field_options   => $options,
                                    empty_options   => $empty,
                                    size            => $size,
                                    help_topic      => 'export',
                                    version         => $VERSION,
                                    },
                                    {
                                        root => $CONFIG{template_dir},
                                        print => 1
                                    });

}

sub html_save_search {

	my $in = shift;

	GT::Template->parse ('save_search.html', { 
                                    do              => scalar $in->param('do'),
                                    home_url        => $CONFIG{'home_url'},
                                    script_url      => $CONFIG{'script_url'}, 
                                    data_source     => scalar $in->param('data_source'),
                                    db              => &get_db($in->param('data_source')),
                                    table           => scalar $in->param('table'),
                                    insert_fields   => $insert_fields,
                                    record_modify   => scalar $in->html_escape($in->param('record_modify')),
                                    page            => scalar $in->param('page') || 1,
                                    action          => scalar $in->param('action'),
                                    sort_index      => scalar $in->html_escape($in->param('sort_index')),
                                    fields          => scalar $in->html_escape($in->param('fields')),
                                    where           => scalar $in->html_escape($in->param('where')), 
                                    example         => scalar $in->html_escape($in->param('example')),
                                    browse_action   => scalar $in->param('browse_action'),
                                    help_topic      => 'save_search_result',
                                    version         => $VERSION,
									query			=> scalar $in->html_escape($in->param('query')) 
                                    },
                                    {
                                        root => $CONFIG{template_dir},
                                        print => 1
                                    });

}

sub html_mysqldump{
# --------------------------------------------------------
# Mysqldump page

    my ($in) = @_;

    $table_checkboxes = &get_checkbox_tables_in_database($in) or return;

    GT::Template->parse ('op_mysqldump.html', {
                                    do              => scalar $in->param('do'),
                                    home_url        => $CONFIG{'home_url'},
                                    script_url      => $CONFIG{'script_url'},
                                    data_source     => scalar $in->param('data_source'),
                                    db              => &get_db($in->param('data_source')),
                                    table           => scalar $in->param('table'),
                                    tb_checkboxes   => $table_checkboxes,
                                    help_topic      => 'sql_dump',
                                    version         => $VERSION,
                                    },
                                    {
                                        root => $CONFIG{template_dir},
                                        print => 1
                                    });

}

sub html_add_fields{
# --------------------------------------------------------
# Add field/column to table page.

    my $in = shift;

    my $options_col_order = qq|\n<option value="">At End of Table</option>\n<option value="FIRST">At Beginning of Table</option>|;
    foreach my $col (&get_cols($in, $in->param('table'))) {
        $options_col_order .= qq|\n<option value="AFTER $col">After $col</option>|;
    }


    GT::Template->parse ('op_add_fields.html', {
                                    do              => scalar $in->param('do'),
                                    home_url        => $CONFIG{'home_url'},
                                    script_url      => $CONFIG{'script_url'},
                                    data_source     => scalar $in->param('data_source'),
                                    db              => &get_db($in->param('data_source')),
                                    table           => scalar $in->param('table'),
                                    opt_col_order   => $options_col_order, 
                                    help_topic      => 'add_col',
                                    version         => $VERSION,

                                    },
                                    {
                                        root => $CONFIG{template_dir},
                                        print => 1
                                    });
}
sub html_rename_table{
# --------------------------------------------------------
# Rename table page.

    my $in = shift;
    GT::Template->parse ('op_rename_table.html', {
                                    do              => scalar $in->param('do'),
                                    home_url        => $CONFIG{'home_url'},
                                    script_url      => $CONFIG{'script_url'},
                                    data_source     => scalar $in->param('data_source'),
                                    db              => &get_db($in->param('data_source')),
                                    table           => scalar $in->param('table'),
                                    help_topic      => 'rename',
                                    version         => $VERSION,
                                    },
                                    {
                                        root => $CONFIG{template_dir},
                                        print => 1
                                    });
}

#=================================================#
#            Show SQL Monitor Query               #
#=================================================#

sub html_show_query{
	
	my $in = shift;
	
	my $i     = $in->param('query_displayed');
	my $query = $in->cookie("query_$i");
	$query =~ s/\n/<BR>\n/g;

	GT::Template->parse ('show_query.html', { query => $query, i => $i},
                                    {
                                        root => $CONFIG{template_dir},
                                        print => 1
                                    });
}

#=================================================#
#                   Help Pages                    #
#=================================================#

sub html_help {
	my $in    = shift;
    my $topic = $in->param("help_topic");
	if ($topic =~ /^\w+\.jpg$/) {
		open (FH, "$CONFIG{template_dir}/help/$topic");
		binmode FH;
		binmode STDOUT;
		while (read (FH, my $buf, 4096)) {
			print $buf;
		}
		close FH;
	}
	else {
		if ($topic !~ /^\w+$/) {
			sqlerr("Invalid help topic: $topic\n");
		}
		GT::Template->parse ('help/help_' . $topic . '.html', { script_url => $CONFIG{script_url} }, { root => $CONFIG{template_dir}, print => 1 });
	}
}

#=================================================#
#                  Demo Prompt                    #
#=================================================#

sub html_demo_prompt{
# --------------------------------------------------------
# Demo message.  It is displayed when demo mode is on and the action performed 
# is disabled.
#
#     demo_message: message specified depending on action performed.


    my ($in, $demo_message) = @_;   
    GT::Template->parse ('demo_prompt.html', {
                                    do              => scalar $in->param('do'),
                                    home_url        => $CONFIG{'home_url'},
                                    script_url      => $CONFIG{'script_url'},
                                    data_source     => scalar $in->param('data_source'),
                                    db              => &get_db($in->param('data_source')),
                                    table           => scalar $in->param('table'),
                                    demo_message    => $demo_message,
                                    help_topic      => 'demo',
                                    version         => $VERSION,
                                    },
                                    {
                                        root => $CONFIG{template_dir},
                                        print => 1
                                    });
    return;
}
1;

