#!/usr/local/bin/perl5.8.8
# ==================================================================
# MySQLMan - web based MySQL manager.
#
#   Website  : http://gossamer-threads.com/
#   Support  : http://gossamer-threads.com/scripts/support/
#   Revision : $Id: mysql.cgi,v 1.50 2002/01/03 01:57:03 alex Exp $
# 
# Copyright (c) 2000 Gossamer Threads Inc.  All Rights Reserved.
# Redistribution in part or in whole strictly prohibited.
# ==================================================================
#
# Auto set the path.  If the path is not set automatically then 
# uncomment the following line and fill in the full path.
#
# use lib '/path/to/MySQLMan';
#
# (You may need change the auto set path part of mysql.cfg as well.)

use strict;
use vars qw($VERSION %CONFIG $SCRIPT_PATH $PRINT_HEADER $DBH);

$VERSION    = '1.09';
$SCRIPT_PATH = '.';

# Required Librariers
# --------------------------------------------------------
# Make sure we are using perl 5.003 and load other required files.
#
check_libraries();

# Run the program and trap fatal errors.
# --------------------------------------------------------
eval { &main; } or cgierr ("fatal undef error: $@", 1); # Trap any fatal errors so the program hopefully 
                                                        # never produces that nasty 500 server error page.
sub main {
# ---------------------------------------------------
# The main program.
#
    $DBH = undef;       # Initilize for mod_perl
    $PRINT_HEADER = 0;  

    my $in = new GT::CGI;

    my $level;
    if (defined ($in->param('db_user')) || defined ($in->param('db_host'))) { &do_login($in); }
    else { 
        $level = $in->param("do") || '';

        if ($level eq "logout") { 
            &do_logout($in); 
            if ( $CONFIG{'debug'} ) {&cgierr("debug");}
            return 1; 
        }
        
        if (($level ne "login") && !$in->param('form_login') ){ &assign_cookies($in); } 
        else { if (!$PRINT_HEADER) { print $in->header(); $PRINT_HEADER=1} }

        if ($CONFIG{'direct_connect'} && !$in->param) {$level = 'tables';}
        
        if (!$level)                      { show_dbs($in); }           # Diplay the database list.
        elsif ($level eq "database")      { modify_db($in); }          # Create or drop a database.
        elsif ($level eq "login")         { html_login($in); }         # Prompt the log-in page when needed.
        elsif ($level eq "tables")        { show_tables($in); }        # Display the list of tables.
        elsif ($level eq "browse")        { table_browse($in); }       # Do a general browse.
        elsif ($level eq "select")        { table_select($in); }       # Compose query criteria for browse.
        elsif ($level eq "insert")        { html_insert($in); }        # Input value for insert.
        elsif ($level eq "insert_record") { insert_record($in); }      # Insert the value input into table.
        elsif ($level eq "property")      { table_property($in); }     # Display column spec's of the current table.
        elsif ($level eq "modify")        { table_modify($in); }       # Modify the table contents.
        elsif ($level eq "create")        { html_table_def($in,'create'); }    # Construct the specifications of the new table.
        elsif ($level eq "create_table")  { create_table($in); }       # Create a new table according to the specification.
        elsif ($level eq "alter_table")   { alter_table($in); }        # Change the structure of a table.
        elsif ($level eq "add_col")       { html_table_def($in,'add_col'); }   # Add new column(s) to a table.
        elsif ($level eq "sql_monitor")   { sql_monitor($in); }        # Process query entered in SQL Monitor.
        elsif ($level eq "sql_monitor_file") { sql_monitor_file($in); }# Process queries saved in a file. 
        elsif ($level eq "import")        { import_record($in); }      # Do import from file.
        elsif ($level eq "export")        { export_record($in); }      # Do export to file.
        elsif ($level eq "mysqldump")     { mysqldump($in); }          # Create a table dump into file specified.
        elsif ($level eq "top_level_op")  { top_level_op($in); }       # Create db/create table/SQL Monitor/import/export/
                                                                        # add fields/rename table.
		elsif ($level eq "show_query")	  { html_show_query($in); }	# display saved query in SQL monitor.
		elsif ($level eq "save_search")	  { html_save_search($in); }
        elsif ($level eq "help")          { html_help($in); }          # display help pages
        else { cgierr("fatal error: $@"); }                            # Display error message if error occurs.
        
        if ($DBH) {$DBH->disconnect();}
    }
    if ( $CONFIG{'debug'} ) {&cgierr("debug");}
    return 1;
}

sub show_dbs {
# ---------------------------------------------------
# Diplays all the databases in MySQL. The function will 
# take the output of SHOW DATABASES query and list all
# the databases in MySQL.  Each name is a link to the table 
# list of the database and a "Drop" link is also created here
# with each database for easy management.

    my ($in, $feedback) = @_;
    my ($db, $dsn, $query, $sth, $db_table, $database_list, $db_table_rows, $drop_cmd);

    if (!$DBH) {&connect_db($in) or return; }
    $query = "SHOW DATABASES";
    $sth = &exec_query($query) or return;
        
    $database_list  = '';
    $drop_cmd       = '';

    while  (($db) = $sth->fetchrow_array) {
        $database_list  = qq~<a href="$CONFIG{'script_url'}?do=tables&data_source=DBI:mysql:$db">$db</a>\n~;
        $drop_cmd       = qq~<a href="$CONFIG{'script_url'}?do=database&data_source=DBI:mysql:$db&action=drop_db&db=$db">Drop</a>\n~;
        $db_table_rows .= qq~<TR><TD>$database_list</TD><TD>$drop_cmd</TD></TR>~;
    }
    $sth->finish;
    $db_table = qq~<TABLE BORDER=2><TR><TH>DATABASE</TH><TH>ACTION</TH></TR>$db_table_rows</TABLE>~;


    &html_database($in, $dsn, $db_table, $feedback) or return;
}

#=================================================#
#              DATABASE MANAGEMENT                #
#=================================================#

sub modify_db{
# ---------------------------------------------------
# Then function will determine whether to create a new 
# database or drop a existing one.

    my $in = shift;
    my $action = $in->param('action') || '';

    if    ($action eq 'drop_db')    { &drop_db($in); }
    elsif ($action eq 'create_db')  { &create_db($in); }
    else  {&cgierr("database modify action cannot be identified.");}
}

sub drop_db{
# ---------------------------------------------------
# Here a "DROP DATABASE db_name" query is executed.  If 
# the confirmed flag is not on then the user will be brought 
# to a confirmation page.  If the action is confirmed, the 
# database specified will be dropped and the user will be brought 
# back to the database list page.
# It is disabled in demo mode.

    my $in = shift;
    my $db = $in->param('db') || '';
    
    my ($query, $sth);

    $query = "DROP DATABASE $db";

    if ( $in->param('comfirmed') ){

        if (!$DBH) {&connect_db($in) or return; }
        $sth = &exec_query($query) or return;
        $sth->finish;
        my $message = "Database $db Dropped.";
        &show_dbs($in, $message);
    }
    else { &html_confirm_action($in, $query); }
}

sub create_db {
# ---------------------------------------------------
# Before a new database is created, the name specified will
# be tested to see if it is a valid one.  If it is, then the 
# table will be created and the user will be brought back to the 
# database list.
# It is disabled in demo mode.

    my $in = shift;
    my $db = $in->param('db') || '';
    
    my ($query, $sth);
    &valid_name_check($db) or return;


    if (!$DBH) {&connect_db($in) or return; }
    $query = "CREATE DATABASE $db";
    $sth = &exec_query($query) or return;
    $sth->finish;

    my $message = "New Database $db Created";
    &show_dbs($in, $message);
}


#=================================================#
#               TABLE MANAGEMENT                  #
#=================================================#


# ================= #
#   Table Display   #
# ================= #

sub show_tables {
# ---------------------------------------------------
# Shows all the tables in the database chosen.  Browse/Select
# /Properties/Insert/Drop/Empty links are also created with 
# each table name.

    my ($in, $feedback) = @_;
    my ($query, $sth, $table_tables, $table);
    my $data_source = $in->param("data_source") || '';

    if (!$DBH) {&connect_db($in) or return; }
    $query = "show tables";
    $sth = &exec_query($query) or return;
    
    $table_tables = '';
    while  (($table) = $sth->fetchrow_array()) {
        $table_tables .= 
        qq~<TR><TD>$table</TD>
            <TD><A href="$CONFIG{'script_url'}?do=browse&data_source=$data_source&table=$table&page=1&action=browse">Browse</A></TD>
            <TD><A href="$CONFIG{'script_url'}?do=select&data_source=$data_source&table=$table&page=1&action=select">Select</A></TD>
            <TD><A href="$CONFIG{'script_url'}?do=property&data_source=$data_source&table=$table">Properties</A></TD>
            <TD><A href="$CONFIG{'script_url'}?do=insert&data_source=$data_source&table=$table">Insert</A></TD>
            <TD><A href="$CONFIG{'script_url'}?do=modify&data_source=$data_source&table=$table&action=drop_table">Drop</A></TD>
            <TD><A href="$CONFIG{'script_url'}?do=modify&data_source=$data_source&table=$table&action=empty_table">Empty</A></TD>
            <TD align="right">~ . &record_count($table) . qq~</TD>
            </TR>\n~;
    }
    $sth->finish();
    &html_table($in, $table_tables, $feedback);
}

# ======================= #
#   Table Browse/Select   #
# ======================= #

sub table_browse{
# ---------------------------------------------------
# Browse, Select/Search
#
#   The function does a "SELECT * FROM table_name" query 
#   to do a browse and will display the results according to 
#   the select criteria specified by user in "select".  If a 
#   primary key exists in the table, then "edit" and "drop" links
#   are also created with each record.
# 
# SQL-Monitor
#
#   A query entered in SQL monitor that requires
#   displaying it's result uses this sub-routine as well.  
#   These queries include explain, select, describe, and desc.

    my ($in, $query) = @_;
    my ($where_clause, $start_row, $empty_set, @pri_key, $prep, $sth, @cols, $rows, @ary, $index,
        $col_name, $table_records, $record, $cells, $page_jump, $page_link, $rows_in_page,
        @fields, @example, @record_modify, $record_modify, $edit_link, @table_list, 
        $table_num, $pri_key_count, $pri_key, $query_printed, $query_count, $total_rec_num);
    
    my $data_source = $in->param("data_source") || '';
    my $table       = $in->param("table")       || '';
    my $page        = $in->param("page")        || "1";
    my $action      = $in->param("browse_action") || $in->param("action") || '';
    my $fields      = $in->param("fields")      || '';
    my $where       = $in->param("where")       || '';
    my $example     = $in->param("example")     || '';


    if (($page =~ m/\D/))     { 
        &sqlerr("Page number cannot be $page.  Please enter a valid page number."); 
        return;
    }

    # The first row of the page arrived.
    $start_row = (($page - 1) * $CONFIG{'page_length'}); 

    if (!$DBH) {&connect_db($in) or return; }

    # get all column names for the table and store them in @cols.
    if ($action eq 'browse' || $action eq 'select') {
        @cols = &get_cols($in, $table);
    }

    if ($action eq 'browse') { 
        $fields = '*'; 
        $index = $in->param("sort_index") || $cols[$CONFIG{'sort_default'}];
    }
    elsif ($action eq 'select') { 

        # SELECT clause.
        if (!$fields) {

            @fields = ();
            for (my $i = 0; $i <= $#cols; $i++) {
                if ($in->param("*select_field*_$cols[$i]") ne '') {
                    push( @fields, $in->param("*select_field*_$cols[$i]") );
                }
            }
            $fields = join ",", @fields;
        }
        
        # WHERE clause.
        if ($where) { $where_clause = "WHERE $where"; }

        # Query by example.
        if (!$example) {
            @example = ();
            for (my $i = 0; $i <= $#cols; $i++) {
                if ($in->param("*example*_$cols[$i]") ne '') {
                    my $temp = $cols[$i] . ' like ' . $DBH->quote( $in->param("*example*_$cols[$i]") );
                    push( @example, $temp );
                }
            }
            $example = join " and ", @example;
        }
    
        if ($example) {
            if ($where_clause) { $where_clause .= " and $example"; }
            else { $where_clause = "WHERE $example"; }
        }
        
        # get sort index
        $query = qq~SELECT $fields
                    FROM $table 
                    $where_clause
                    LIMIT 1~;

        $prep = &exec_query($query) or return;
        $index = $in->param("sort_index") || $prep->{NAME}->[$CONFIG{'sort_default'}];
        $prep->finish();
    }

    @pri_key = ();
    $pri_key = '';

    if ($action eq 'browse' || $action eq 'select') {
        # Prepare the contents in the table selected.
    
        @pri_key = &get_pri_key($table);
        $pri_key = join ",", @pri_key;

        if ($pri_key ne '') { $pri_key = ',' . $pri_key; }
        
        # the actual query sent to statement handler.  The 
        # primary key is selected for delete and edit.
        $query = qq~SELECT $fields $pri_key 
                    FROM $table 
                    $where_clause   
                    ORDER BY $index 
                    LIMIT $start_row, $CONFIG{'page_length'}~;
        
        # The query that gets printed in SQL-Message.
        $query_printed = qq~SELECT $fields
                            FROM $table 
                            $where_clause   
                            ORDER BY $index 
                            LIMIT $start_row, $CONFIG{'page_length'}~;

        # counts the total number of resulting records from browse/select
        $query_count =qq~SELECT COUNT(*) 
                         FROM $table 
                         $where_clause~;
        $sth = &exec_query($query_count) or return;
        ($total_rec_num) = $sth->fetchrow();
        $sth->finish();
    }
    else{ # from SQL Monitor.
        if (!$query) { $query = $in->param("query") || ''; }
        @table_list = &get_table_list($query);
        if ($#table_list == 0) { ($table) = @table_list; }
    }
    
    # Get all records or records that satisfy the search criteria.
    # Or, $query is simply the query enter in SQL monitor.
    $sth = &exec_query($query) or return;
    $rows = $sth->rows;
    
    if ($action eq 'monitor') { 
        $total_rec_num = $rows;
        $rows_in_page = $rows - $start_row;
        if ($rows_in_page > $CONFIG{'page_length'}) { $rows_in_page = $CONFIG{'page_length'} }
    }
    else { $rows_in_page = $rows; }

    $page_jump = &link_page_jump($in, $fields, $example, $index, $total_rec_num);

    # Links to next/previous/top page if there is any.
    $page_link = &link_page($in, $rows_in_page, $table, $fields, $example, $index, $total_rec_num);
    
    my $example_esc = $in->escape($example);
    my $where_esc   = $in->escape($where);
    my $query_esc   = $in->escape($query);

    if (@pri_key != ()) { $pri_key_count = $#pri_key + 1; }
    else {$pri_key_count = 0; }

    # Display column names
    $col_name = '';
    if ($action eq 'browse' || $action eq 'select') {
        for (my $i = 0; $i < $sth->{NUM_OF_FIELDS}- $pri_key_count; $i++) {
            $col_name .= qq~\n<TD><B><a href= "$CONFIG{'script_url'}?do=browse&data_source=$data_source&table=$table&page=$page&sort_index=$sth->{NAME}->[$i]&action=$action&fields=$fields&where=$where_esc&example=$example_esc&query=$query_esc">$sth->{NAME}->[$i]</B></TD>~;
        }
    }
    else {
        for (my $i = 0; $i < $sth->{NUM_OF_FIELDS}- $pri_key_count; $i++) {
            $col_name .= qq~\n<TD><B>$sth->{NAME}->[$i]</B></TD>~;
        }
    }

    $example_esc = $in->html_escape($example);
    $where_esc   = $in->html_escape($where);
    $query_esc   = $in->html_escape($query);

    # Display the contents in the table selected.
    $table_records = '';
    my $counter = 0;

    # Display the resulting set of records.  The result is divided into pages and the page
    # by the length specified in mysql.cfg.
    #
    # From "Browse" or "Select/Search"
    #     The LIMIT clause of the query limits the records to display, so we don't display all.
    # From "SQL Monitor"
    #     Records between $start_row and $start_row+$CONFIG{'page_length'} will be displayed.
    #
    while ( @ary = $sth->fetchrow_array() ) {
        if ($action ne 'monitor' || ($counter >= $start_row && $counter < ($CONFIG{'page_length'} + $start_row))) {
            $record = '';
            @record_modify = ();

            for (my $i = 0; $i < $sth->{NUM_OF_FIELDS}; $i++) {
                if ($i < $sth->{NUM_OF_FIELDS} - $pri_key_count) { 

                    $ary[$i] =~ s/(\r|\n)+/<BR>/g;
                    $ary[$i] =~ s/\t+/&nbsp;&nbsp;&nbsp;/g;
                    $ary[$i] =~ s/\s+/&nbsp;/g;
                    $ary[$i] =~ s/<BR>/<BR>\n/g;

                    if (defined($ary[$i])) { 
                        if ($ary[$i] ne '') { $record .= "<TD>$ary[$i]</TD>"; }

                        # put a space in the cell for display purpose.
                        else { $record .= "<TD>&nbsp;</TD>"; }
                    }
                    else { 
                        if ($CONFIG{'show_null'}) { $record .= "<TD><FONT COLOR=green>NULL</FONT></TD>"; }
                        else { $record .= "<TD>$ary[$i]</TD>"; }
                    }
                }

                # if there are any primary keys, then push "pri = value"
                # into @record_modify.
                else{
                    my $cell = $ary[$i];
                    $cell = $DBH->quote($cell);
                    my $col_name = $sth->{NAME}->[$i];
                    push (@record_modify,  $col_name . ' = ' . $cell);
                }
            }
            if ((@pri_key != ()) && ($action ne 'monitor')) {
                                
                $record_modify = join ' and ', @record_modify;
                $record_modify = $in->escape($record_modify);

                my ($edit_link, $delete_link);

				$edit_link = qq~<a href= "$CONFIG{'script_url'}?do=modify&data_source=$data_source&table=$table&page=$page&sort_index=$index&fields=$fields&where=$where_esc&example=$example_esc&action=edit_record&record_modify=$record_modify&browse_action=$action">Edit~;
				if ($CONFIG{'confirm_delete_record'}) {
					$delete_link = qq~<a href= "$CONFIG{'script_url'}?do=modify&data_source=$data_source&table=$table&page=$page&sort_index=$index&fields=$fields&where=$where_esc&example=$example_esc&action=delete_record&record_modify=$record_modify&browse_action=$action" onClick="return confirm('Delete the record?')">Delete~;
				}
				else{
					$delete_link = qq~<a href= "$CONFIG{'script_url'}?do=modify&data_source=$data_source&table=$table&page=$page&sort_index=$index&fields=$fields&where=$where_esc&example=$example_esc&action=delete_record&record_modify=$record_modify&browse_action=$action">Delete~;
				}
                $table_records .= 
                qq~<TR>
                $record
                <TD>$edit_link</TD>
                <TD>$delete_link</TD>
                </TR>\n~;
            }
            else { $table_records .= qq~<TR>$record</TR>\n~; }              
        }
        $counter++;
    }
    if (!$record) { $empty_set = 1; }
    else { $empty_set = 0; }

    $sth->finish();

	my $save_search_link = "$CONFIG{'script_url'}?do=save_search&data_source=$data_source&table=$table&page=$page&sort_index=$index&fields=$fields&where=$where_esc&example=$example_esc&action=edit_record&record_modify=$record_modify&browse_action=$action&query=$query_esc";

    &html_table_browse($in, $table, $page_jump, $page_link, $col_name, $table_records, $query, $empty_set, $pri_key, $query_printed, $total_rec_num, $save_search_link);
}

sub table_select {
# ---------------------------------------------------
# This function creates the search form for a SELECT... query (search).
# Field names are in check boxes and "query by example" input 
# fields are created with the check boxes.

    my $in = shift;
    my ($query, $select_fields, $example_fields, $select_table, $example_table, @type_ary, @cols);
    
    my $data_source = $in->param("data_source") || '';
    my $table       = $in->param("table")       || '';
    my $page        = $in->param("page")        || 1;

    $select_fields  = '';
    $example_fields = '';


    if (!$DBH) { &connect_db($in) or return; }

    # Get the names of the all columns.
    @cols = &get_cols($in, $table);

    # fields selection (for SELECT)
    @type_ary = &get_col_type($table); 
    for (my $i = 0; $i <= $#cols; $i++) {
        
        # make the first field checked to avoid error message due to empty select clause.
        if ($i==0) { $select_fields .= qq~\n<input type="checkbox" name="*select_field*_$cols[$i]" value="$cols[$i]" checked>$cols[$i]<BR>~; }
        else       { $select_fields .= qq~\n<input type="checkbox" name="*select_field*_$cols[$i]" value="$cols[$i]">$cols[$i]<BR>~; }
        
        # output the check boxes such that there are 5 rows in a column.
        if ( ($i+1)%5 == 0 ) { $select_fields .= "</TD><TD VALIGN = top>"; }
    
        # compose "query by example" input fields.
        $example_fields .= qq~\n<TR><TD>$cols[$i]</TD><TD>$type_ary[$i]</TD><TD><INPUT TYPE="text" NAME="*example*_$cols[$i]" VALUE="" SIZE="30"></TD></TR>~;
    }   

    $select_table = "<TABLE><TR><TD VALIGN = top>" . $select_fields . "</TD></TR></TABLE>";
    $example_table = "<TABLE BORDER = 1><TR><TH>Fields</TH><TH>Type</TH><TH>Value</TH></TR>" . $example_fields . "</TABLE>";
    
    &html_table_select($in, $select_table, $example_table);
}

sub table_property{
# ---------------------------------------------------
# The function outputs the result of "describe table_name"
# query.  It reads the output row by row and create 
# Change/Drop/Primary/Index/Unique links with each field.

    my ($in, $feedback) = @_;
    my ($page, $query, $sth, @ary, $table_property, $table_property_row);

    my $data_source = $in->param("data_source") || '';
    my $table       = $in->param("table")       || '';


    if (!$DBH) { &connect_db($in) or return; }
    $query = "describe $table";
    $sth = &exec_query($query) or return;

    # Display the contents in the table selected.
    while ( @ary = $sth->fetchrow_array() ) {
        $table_property_row .= "\n<TR><TD>" . join ( "</TD><TD>" , @ary) . "</TD>";
        $table_property_row .= qq~\n\t<TD><A href="$CONFIG{'script_url'}?do=alter_table&data_source=$data_source&table=$table&col=$ary[0]&action=alter_col">Change</A></TD>~;
        $table_property_row .= qq~\n\t<TD><A href="$CONFIG{'script_url'}?do=alter_table&data_source=$data_source&table=$table&col=$ary[0]&action=drop_col">Drop</A></TD>~;
        $table_property_row .= qq~\n\t<TD><A href="$CONFIG{'script_url'}?do=alter_table&data_source=$data_source&table=$table&col=$ary[0]&action=set_primary">Primary</A></TD>~;
        $table_property_row .= qq~\n\t<TD><A href="$CONFIG{'script_url'}?do=alter_table&data_source=$data_source&table=$table&col=$ary[0]&action=set_index">Index</A></TD>~;
        $table_property_row .= qq~\n\t<TD><A href="$CONFIG{'script_url'}?do=alter_table&data_source=$data_source&table=$table&col=$ary[0]&action=set_unique">Unique</A></TD></TR>~;
    }
    $sth->finish();

    # for display purpose, each empty cell is replace with a space.
    $table_property_row =~ s/<TD><\/TD>/<TD>\&nbsp\;<\/TD>/g;

    $query = "describe $table";
    $sth = &exec_query($query) or return;
    
    # Construct the table.
    $table_property  = "<TABLE BORDER=2>";
    $table_property .= "<TR><TD><B>". join ("</TD><TD><B>", @{$sth->{NAME}}) . "</TD><TH colspan=5>ACTION</TH></TR>\n";
    $table_property .= $table_property_row;
    $table_property .= "</TABLE>";
    $sth->finish();
    
    &html_property($in, $table_property, $feedback);
}


# ===================== #
#   Insert New Record   #
# ===================== #

sub insert_record{
# ---------------------------------------------------
# This function insert a new record into the table specified.

    my $in = shift;
    my $table = $in->param('table') || '';
    my $feedback;
    my ($query, $sth, @insert_info, $new_record, @insert_fields, $fields, @insert_values, $values);


    if (!$DBH) {&connect_db($in) or return; }
    
	# Get the info of the new record to be inserted.
    @insert_info = &compose_new_condition($in, 1);

	my $counter = 0;
	foreach my $element (@insert_info) {
		
		my $is_value = $counter % 2;

		if ($is_value) { # Get the values
			push (@insert_values, $element)
		}
		else { # Get the name of the fields
			push (@insert_fields, $element)
		}
		$counter++
	}

	# Make the input from the form into a string to fit in the query.
    $fields = join ",", @insert_fields;
	$values = join ",", @insert_values;

    $query = "INSERT INTO $table ($fields) VALUES ($values)";  
    $sth = &exec_query($query) or return;
    $sth->finish();
    
    $feedback = 'Record Inserted.';
    if ($CONFIG{'insert_origin'} eq 'table') { &show_tables($in, $feedback); }
    else { &html_insert($in, $feedback) }
}

# ====================== #
#    Create New Table    #
# ====================== #

sub create_table{
# ---------------------------------------------------
# This function takes in the input from the create table 
# form and put them together to produce a create table 
# query.

    my $in = shift;
    my $table = $in->param('table') || '';
    my (@field_list, $col_spec, @primary_list, @index_list, @unique_list, $fields, $primary, $index, $unique, $sth, $query);


    if (!$DBH) {&connect_db($in) or return; }

    # get the specification of each column.
    for (my $i = 0; $i < $in->param('num_of_fields'); $i++) {

        # Make the text input fields into a string to fit in the query.
        $col_spec = &concate_col_spec($in, $i);
        
        push (@field_list, "$col_spec");
        
        # Check index fields.
        if ( $in->param("primary_$i") ) { push (@primary_list, $in->param("field_$i")); }
        if ( $in->param("index_$i") )   { push (@index_list, $in->param("field_$i")); }
        if ( $in->param("unique_$i") )  { push (@unique_list, $in->param("field_$i")); }
    }
    
    $fields  = join ",", @field_list;
    $primary = join ",", @primary_list;
    $index   = join ",", @index_list;  
    $unique  = join ",", @unique_list;
    
    $query = "CREATE TABLE $table($fields";

    if ($primary){ $query .= ", PRIMARY KEY ($primary)" }
    if ($index)  { $query .= ", INDEX ($index)" }
    if ($unique) { $query .= ", UNIQUE ($unique)" }

    $query .= ')';
    
    $sth = &exec_query($query) or return;
    $sth->finish();
    
    &show_tables($in, "Table $table Created.");
}

# ====================== #
#   Table Modification   #
# ====================== #

sub table_modify{
# ---------------------------------------------------
# Determine modify action.

    my $in = shift;
    my $action = $in->param('action') || '';

    if      ($action eq 'drop_table')   { &drop_table($in); }
    elsif   ($action eq 'empty_table')  { &empty_table($in); }
    elsif   ($action eq 'delete_record'){ &delete_record($in); }
    elsif   ($action eq 'edit_record')  { &edit_record_html($in); }
    elsif   ($action eq 'update')       { &update_record($in) }
    else    { &cgierr("modify action cannot be identified"); }
}


sub drop_table{
# ---------------------------------------------------
# The function drops the table specified if the confirmed
# flag is on.

    my $in = shift;
    my $table = $in->param('table') || '';
    my ($query, $sth);


    $query = "DROP TABLE $table";

    if ( $in->param('comfirmed') ){
        if (!$DBH) {&connect_db($in) or return; }     
        $sth = &exec_query($query) or return;
        &show_tables($in, "Table $table dropped!")
    }
    else { &html_confirm_action($in, $query); }
}

sub empty_table{
# ---------------------------------------------------
# The function deletes all the records in the table specified 
# if the confirmed flag is on.
    
    my $in = shift;
    my ($table, $query, $sth);

    $table = $in->param('table') || '';
    

    $query = "DELETE FROM $table";

    if ( $in->param('comfirmed') ){
        if (!$DBH) {&connect_db($in) or return; }
        $sth = &exec_query($query) or return;
        &show_tables($in, "Table $table emptied!")
    }
    else { &html_confirm_action($in, $query); }
}

sub delete_record{
# ---------------------------------------------------
# Delete a single record in the table selected.
# $record_modify consists the primary key(s) value of 
# the record being deleted.

    my $in = shift;
    my $table = $in->param('table') || '';
    my $record_modify = $in->param('record_modify') || '';
    my ($sth, $query);


    if (!$DBH) {&connect_db($in) or return; }
    $query = "DELETE FROM $table WHERE $record_modify LIMIT 1";
    $sth = &exec_query($query) or return;
    $sth->finish();
    
    &table_browse($in);
}

sub edit_record_html{
# ---------------------------------------------------
# Pre-processing stage before the edit record form is 
# displayed.  This function prepares necessary information
# for the edit form.
# $record_modify consists the primary key(s)
# value of the record being edited.
# i.e. key = value.

    my $in = shift;
    my $table = $in->param('table') || '';
    my $record_modify = $in->param('record_modify') || '';
    my ($sth, $query, @record, $update, $update_fields);

    
    if (!$DBH) {&connect_db($in) or return; } 
    
    # Get the record being modified.
    $query = "SELECT * FROM $table WHERE $record_modify";   
    $sth = &exec_query($query) or return;
    @record = $sth->fetchrow_array;
    my $row = $sth->rows;
    $sth->finish();
    
    # create the the edit record form table.
    $update_fields = &form_fields($in, 1, @record);
    &html_update($in, $update_fields);
}

sub update_record{
# ---------------------------------------------------
# Take in the input from the edit table form and update 
# the record specified.

    my $in = shift;
    my $table = $in->param('table') || '';
    my $record_modify = $in->param('record_modify') || '';
    my ($sth, $query, $update, @fields);


    if (!$DBH) {&connect_db($in) or return; }
    
    # Get the updated values in each field.  Each element in the 
    # field is in the form "field = value".
    @fields = &compose_new_condition($in);

    $update = join ",", @fields;
    $query = "UPDATE $table SET $update WHERE $record_modify";  
    $sth = &exec_query($query) or return;
    $sth->finish();
    
    &table_browse($in);
}

# ====================== #
#    Table Alteration    #
# ====================== #

sub alter_table{
# ---------------------------------------------------
# Identify alter table action.

    my $in = shift;
    my $action = $in->param('action') || '';

    if      ($action eq 'alter_col')    { &alter_col_html($in); }
    elsif   ($action eq 'do_alter_col') { &alter_col($in); }
    elsif   ($action eq 'drop_col')     { &drop_col($in); }
    elsif   ($action eq 'set_primary')  { &set_primary($in); }
    elsif   ($action eq 'set_index')    { &set_index($in); }
    elsif   ($action eq 'set_unique')   { &set_unique($in); }
    elsif   ($action eq 'drop_key')     { &drop_key($in); }
    elsif   ($action eq 'add_col')      { &add_col($in);}
    elsif   ($action eq 'rename_table') { &rename_table($in);}
    else    { &cgierr("Alter Table action cannot be idenfied"); }
}

sub alter_col_html{
# ---------------------------------------------------
# The function first reads in the spec's of the column 
# chosen in the current table.  Then the type/length_set
# /attribute is identified individually.

    my $in = shift;
    my $table = $in->param('table') || '';
    my $col = $in->param('col')     || '';

    my ($field, $type, $null, $key, $default, $extra);
    my ($sth, $query, $length_set, $attributes, $dump, $type_name);

    
    # Get column specification.
    if (!$DBH) {&connect_db($in) or return; }
    $col = $DBH->quote($col);
    $query = "SHOW COLUMNS FROM $table LIKE $col";
    $sth = &exec_query($query) or return;
    
    # parse column definition.
    my @ary = $sth->fetchrow_array;
    ($field, $type_name, $length_set, $attributes, $null, $key, $default, $extra) = parse_col_spec(@ary);
    $sth->finish();

    &html_alter_col($in, $field, $type_name, $length_set, $attributes, $null, $default, $extra);
}

sub alter_col{
# ---------------------------------------------------
# Updates the column specification.  The input from the 
# alter column is taken in and made into a string to be
# fit as part of the query string.  Then the user is 
# brought back to the property page.

    my $in = shift;
    my $table = $in->param('table') || '';
    my $col = $in->param('col')     || '';
    my ($col_spec, $sth, $query);


    if (!$DBH) {&connect_db($in) or return; }

    # Get the updated column specs in string format.
    $col_spec = &concate_col_spec($in, 0);
    
    $query = "ALTER TABLE $table CHANGE $col $col_spec";
    $sth = &exec_query($query) or return;
    $sth->finish();
    
    &table_property($in, "Specification of Column $col of Table $table Has Been Changed.")
}

sub drop_col{
# ---------------------------------------------------
# The function drops the column/field specified 
# if the confirmed flag is on.

    my $in = shift;
    my $table = $in->param('table') || '';
    my $col = $in->param('col')     || '';
    my ($col_spec, $sth, $query);


    $query = "ALTER TABLE $table DROP $col";

    if ( $in->param('comfirmed') ){
        if (!$DBH) {&connect_db($in) or return; }
        $sth = &exec_query($query) or return;
        $sth->finish();
        &table_property($in, "Column $col of Table $table Has Been Dropped.");
    }
    else { &html_confirm_action($in, $query); }
}

sub set_primary{
# ---------------------------------------------------
# The function will first set the column not nullable 
# and then set the column as primary key.  Note that an
# error will occur if all there already exists a primary key.

    my $in = shift;
    my $table = $in->param('table') || '';
    my $col = $in->param('col')     || '';
    my ($sth, $query);


    if (!$DBH) {&connect_db($in) or return; }
    $query = "ALTER TABLE $table ADD PRIMARY KEY ($col)";
    
    # Set the column not nullable
    &set_col_not_null($in);

    $sth = &exec_query($query) or return;
    $sth->finish();
    
    &table_property($in, "Column $col set as primary key.");
}

sub set_index{
# ---------------------------------------------------
# The function will first set the column not nullable 
# and then set the column as index.  Note that an
# error will occur if all there already exists a primary key.

    my $in = shift;
    my $table = $in->param('table') || '';
    my $col = $in->param('col')     || '';
    my ($sth, $query);

    
    $query = "ALTER TABLE $table ADD INDEX ($col)";

    if (!$DBH) {&connect_db($in) or return; }

    # Set the column not nullable.
    &set_col_not_null($in);
    
    $sth = &exec_query($query) or return;
    $sth->finish();
    
    &table_property($in, "Column $col set as index.");
}

sub set_unique{
# ---------------------------------------------------
# The function will first set the column not nullable 
# and then set the column as unique.  Note that an
# error will occur if all there already exists a primary key.

    my $in = shift;
    my $table = $in->param('table') || '';
    my $col = $in->param('col')     || '';
    my ($sth, $query);


    $query = "ALTER TABLE $table ADD UNIQUE ($col)";

    if (!$DBH) {&connect_db($in) or return; }

    # Set the column not nullable.
    &set_col_not_null($in);
    
    $sth = &exec_query($query) or return;
    $sth->finish();
    
    &table_property($in, "Column $col set as unique.");
}

sub drop_key{
# ---------------------------------------------------
# Drops the key specified.
    
    my $in = shift;
    my $table = $in->param('table')       || '';
    my $key_name = $in->param('key_name') || '';
    my ($sth, $query);

    
    if ( $key_name eq 'PRIMARY') { $query = "ALTER TABLE $table DROP PRIMARY KEY"; }
    else { $query = "ALTER TABLE $table DROP INDEX $key_name"; }

    if (!$DBH) {&connect_db($in) or return; }
    $sth = &exec_query($query) or return;
    $sth->finish();
    
    if ( $key_name eq 'PRIMARY' ) { &table_property($in, "Primary Key of Table $table Has Been Dropped."); }
    else { &table_property($in, "Index $key_name of Table $table Has Been Dropped."); }
}

sub add_col{
# ---------------------------------------------------
# Adds new columns to the table specified.

    my $in       = shift;
    my $table    = $in->param('table')    || '';
    my $position = $in->param('position') || '';
    my (@field_list, $col_spec, @primary_list, @index_list, @unique_list, $fields, $primary, $index, $unique, $sth, $query);


    if (!$DBH) {&connect_db($in) or return; }

    for (my $i = 0; $i < $in->param('num_of_fields'); $i++) {

        $col_spec = 'ADD ' . &concate_col_spec($in, $i) . " $position";
        push (@field_list, "$col_spec");
        
        if ( $in->param("primary_$i") ) { push (@primary_list, $in->param("field_$i")); }
        if ( $in->param("index_$i") )   { push (@index_list, $in->param("field_$i")); }
        if ( $in->param("unique_$i") )  { push (@unique_list, $in->param("field_$i")); }

        $position = "After " . $in->param("field_$i");
    }
    
    # elements in @field_list are in the form "ADD field_name field_spec".
    $fields  = join ",", @field_list;
    $primary = join ",", @primary_list;
    $index   = join ",", @index_list;  
    $unique  = join ",", @unique_list;

    $query = "ALTER TABLE $table $fields";

    if ($primary){ $query .= ", ADD PRIMARY KEY ($primary)" }
    if ($index)  { $query .= ", ADD INDEX ($index)" }
    if ($unique) { $query .= ", ADD UNIQUE ($unique)" }
    
    
    $sth = &exec_query($query) or return;
    $sth->finish();
    
    &table_property($in, "Column(s) added to Table $table");
}

sub rename_table{
# ---------------------------------------------------
# Renames the table chosen to a new name specified.
# The name entered will be checked to see if it is a 
# valid one.  If it is, then the table will be renamed
# and the user will be brought back to the table property 
# page.

    my $in = shift;
    my $new_name  = $in->param('table')     || '';
    my $old_table = $in->param('old_table') || '';
    my ($query, $sth, @name);

    &valid_name_check($in->param('table')) or return;


    if (!$DBH) {&connect_db($in) or return; }
    $query = "ALTER TABLE $old_table RENAME AS $new_name";
    $sth = &exec_query($query) or return;
    $sth->finish();

    &table_property($in, "Table $old_table Renamed to $new_name.");
}

#=================================================#
#             Top Level Operations                #
#=================================================#

sub top_level_op{
# ---------------------------------------------------
# Determine which top level operation page to display.

    my $in = shift;
    my $action = $in->param('action') || '';
    if      ($action eq 'create_db')    {&html_create_db($in)}
    elsif   ($action eq 'sql_monitor')  {&html_sql_monitor($in)}
    elsif   ($action eq 'create_table') {&html_create_table($in)}
    elsif   ($action eq 'import')       {&html_import($in, &table_field_prep($in))}
    elsif   ($action eq 'export')       {&html_export($in, &table_field_prep($in))} 
    elsif   ($action eq 'add_fields')   {&html_add_fields($in)} 
    elsif   ($action eq 'rename_table') {&html_rename_table($in)} 
    elsif   ($action eq 'mysqldump')    {&html_mysqldump($in)} 
    else {&cgierr("Action cannot be identified in top level operation.")}
}

#=================================================#
#                 SQL Monitor                     #
#=================================================#

sub sql_monitor{
# ---------------------------------------------------
# The monitor will be enabled when a database is selected
# from the database list page.  It will take in the input 
# from the text box and send it to mysql.  The query will 
# first be determined if it is of a "browse" one.  If it is, 
# the result will be displayed using &table_browse.
# It is disabled in demo mode.

    my $in = shift;
    my $queries = $in->param('query') || '';
	
	if ($in->param('from_saved_queries')) { $queries = &get_pre_query($in) || return &sqlerr("No query specified.");}
	
	my (@query, @lines, $query, $command, $sth, $rows, $message);
    my $query_no = 0;


# What type of queries prompt a browse mode.
    my %browse_cmd = (
        explain  => 1,
        select   => 1,
        show     => 1,
        describe => 1,
        desc     => 1,
    );

	if (!$in->param('save_query')){
        
        @lines = split /[\n\r]/, $queries;

        for (my $i=0; $i<=$#lines; $i++) {
            
            # strips out the beginning and ending spaces.
            $lines[$i] =~ s/^\s+//;  
            $lines[$i] =~ s/\s+$//;

            if (!($lines[$i] =~ /^#/)) {

                if (($lines[$i] =~ /;\s*$/) or ($i == $#lines) ) {
    
                    $lines[$i] =~ s/\s*;\s*$//;
                    $query .= "$lines[$i]";
                    $query =~ s/^\s+//;  
                    $query =~ s/\s+$//;
                    
                    if ($query ne '') {
                        @query = split / /, $query;
                        $command = lc($query[0]);
            
                        # display the result if the query of a browse one.
                        # else execute the query and return the number of 
                        # rows affect.
                                        
                        if ($browse_cmd{$command}) { return &table_browse($in, $query); }
                        else{
                            if (!$DBH) {&connect_db($in) or return; }
							$rows = $DBH->do($query) or return &sqlerr("$DBI::errstr. <P>Query: $query");
                            $rows += 0;
                            $message = "$rows row(s) affected";
                        }
                        $query_no++;
                        $query = '';
                    }
                }
                else {
                    $query .= " $lines[$i]";
                }
            }
        }
        if ($query_no > 1) { $message = 'Queries executed successfully.'}
		return &html_sql_monitor($in, $message);
    }
}


sub sql_monitor_file{
# ---------------------------------------------------
# Does the same thing as sub sql_monitor only that the 
# queries are read from a file.  The functions first checkes 
# if a server file path is entered.  If not, then it will look
# for the file uploaded to the temp directory from local drive. 


    my ($in) = @_;
    my $file = $in->param('server_file');
    my ($data);

    my $query = '';

    if (!$file) { $file = &create_temp_file($in) or return; }


    if (!$DBH) {&connect_db($in) or return; }

    open QF, "$file" or &cgierr("Cannot open query file '$file': $!");
    while (defined (my $line = <QF>)) {
        
        my $isComment = 0;
        my $isBlankLine = 0;
        
        if ($line =~ /^\s*#/)  {$isComment   = 1}
        if ($line =~ /^\s*\n/) {$isBlankLine = 1}
        if (!$isComment and !$isBlankLine) { $query .= " $line"; }

        if ($line =~ /;\s*(\n|$)/) {

            if (!$DBH) {&connect_db($in) or return; }

            # strips out the beginning and ending spaces.
            $query =~ s/^\s+//;  
            $query =~ s/\s+$//;
            $query =~ s/(\r|\n)+/ /g;
            
            if ($query ne ';') {
                # run query.
                $DBH->do($query) or return &sqlerr("$DBI::errstr. <P>Query: $query");
            }
            # clean up query and prepare to take in the next one.
            $query = '';
        }
    }
    close QF;
    if ($in->param('upload_local_file') && !$in->param('server_file')) {
        unlink $file;
    }

    # In case the last query does not end with a semi-colon.
    $query =~ s/^\s+//;  
    $query =~ s/\s+$//;
    $query =~ s/(\r|\n)+/ /g;
    
    if (!($query =~ /;$/) and ($query ne '')) {
        # run query.
        $DBH->do($query) or return &sqlerr("$DBI::errstr. <P>Query: $query");
    }

    &html_sql_monitor($in, "Queries executed successfully.");
}

#=================================================#
#               Import / Export                   #
#=================================================#

sub import_record{
# ---------------------------------------------------
# Import records to the table specified from a delimited 
# text file.
# It is disabled in demo mode.

    my $in = shift;
    my $delimiter       = defined ($in->param('delimiter')) ? $in->param('delimiter') : '';
    my $rec_del         = defined ($in->param('rec_del')) ? $in->param('rec_del') : '';
    my $table           = $in->param('table')       || '';
    my $file            = $in->param('server_file')     || '';
    my $import_all_cols = $in->param('import_all_cols') || '';
    my $local           = $in->param('local')       || '';
    my $replace_op      = $in->param('replace_op')  || '';
    my $replace_act     = $in->param('replace_act') || '';
    my $escape_char     = defined ($in->param('escape_char')) ? $in->param('escape_char') : '';
    my $ignore_line     = $in->param('ignore_line') || 0;

    my @select_fields = $in->param('ImportRight');

    my ($query, $sth, $file_q, $delimiter_q, $rec_del_q, $escape_char_q, $field_op);


    if (!$DBH) {&connect_db($in) or return; }

    if (!$file) {
        $file = &create_temp_file($in) or return;
    }

    # quote the inputs
    $file_q        = $DBH->quote($file);
    $delimiter_q   = "'" . $delimiter . "'";
    $rec_del_q     = "'" . $rec_del . "'";
    $escape_char_q = $DBH->quote($escape_char);

    if (!$replace_op) { $replace_act = ''; }

    $query = qq~LOAD DATA $local INFILE $file_q $replace_act
                INTO TABLE $table
                FIELDS
                    TERMINATED BY $delimiter_q
                    ESCAPED BY $escape_char_q
                LINES TERMINATED BY $rec_del_q
                IGNORE $ignore_line LINES~;
    
    if (!$import_all_cols) {  # import selected fields only.
        my $selected_cols = "(" . join("," , @select_fields) . ")";
        $query .= " $selected_cols"; 
    }

    $sth = &exec_query($query) or return;
    $sth->finish();

    if ($in->param('upload_local_file') && !$in->param('server_file')) {
        unlink $file;
    }

    &show_tables($in, "File Imported Successfully.");
}

sub export_record{
# ---------------------------------------------------
# Exort records from the table specified and produce a 
# delimited text file.
# It is disabled in demo mode.

    my $in = shift;
    my $export_all_cols = $in->param('export_all_cols') || '';
    my $delimiter       = defined ($in->param('delimiter')) ? $in->param('delimiter') : '';
    my $rec_del         = defined ($in->param('rec_del')) ? $in->param('rec_del') : '';
    my $table           = $in->param('table') || '';
    my $file            = $in->param('file') || '';
    my $escape_char     = defined ($in->param('escape_char')) ? $in->param('escape_char') : '';
    my $to_screen       = $in->param('export_to_screen') || '';
    if ($to_screen) { $file = &get_temp_file_name(); }

    my ($query, $sth, $file_q, $delimiter_q, $rec_del_q, $escape_char_q, $cols);

    my @select_fields = $in->param('ImportRight');


    if (!$to_screen && !$file) {
        &sqlerr("Please provide a file name for export.");
        return;
    }

    if (!$DBH) {&connect_db($in) or return; }

    # quote the parameters.
    $file_q        = $DBH->quote($file);
    $delimiter_q   = $DBH->quote($delimiter);
    $rec_del_q     = "'" . $rec_del . "'";
    $escape_char_q = $DBH->quote($escape_char);

    if ($export_all_cols) { # select all fields
        $cols = '*'
    }
    else{ # export selected fields only.
        $cols = join("," , @select_fields);
    }
    
    if ($in->param('from_save_result')) {

        my $fields = $in->param('fields');
        my $action = $in->param('browse_action');

        if ($action eq 'browse') {
            $query = qq~SELECT *
                        INTO OUTFILE $file_q 
                        FIELDS
                            TERMINATED BY $delimiter_q
                            ESCAPED BY $escape_char_q
                        LINES TERMINATED BY $rec_del_q
                        FROM $table~; 
        }
        else {
            
            my $where_clause = '';
            my $where   = $in->param('where');
            my $example = $in->param('example');

            if ($where) { $where_clause = "WHERE $where"; }

            # Query by example.
            if ($example) {
                if ($where_clause) { $where_clause .= " AND $example"; }
                else { $where_clause = "WHERE $example"; }
            }
            
            $query = qq~SELECT $fields
                        INTO OUTFILE $file_q 
                        FIELDS
                            TERMINATED BY $delimiter_q
                            ESCAPED BY $escape_char_q
                        LINES TERMINATED BY $rec_del_q
                        FROM $table 
                        $where_clause~;
        }
    }
    else{
        $query = qq~SELECT $cols
                    INTO OUTFILE $file_q 
                    FIELDS
                        TERMINATED BY $delimiter_q
                        ESCAPED BY $escape_char_q
                    LINES TERMINATED BY $rec_del_q
                    FROM $table~;
    }
    $sth = &exec_query($query) or return;
    $sth->finish();

    if ($to_screen) {        
        print "<pre># Exported data from table $table\n";
        print "# A temporary file ($file) was create in the temp directory.\n";
        print "# Please remove the file manually as necessary.\n\n";
        print "# =========== Export Starts ===========\n";
        open (TEMP, $file) or &cgierr("error open file");
        while (my $line = <TEMP>) {
            print $line;
        }
        close TEMP;
        print "\n# =========== Export Ends ===========</pre>";

        unlink($file);
        return 1;
    }
    elsif ($in->param('from_save_result')){
        &table_browse($in);
    }
    else{
        return &show_tables($in, "File Exported Successfully.");
    }
}

sub table_field_prep{
# ---------------------------------------------------
# Create options for import and export HTML pages.  The
# options are the fields of the table selected.

    my($in) = @_;
    my $table = $in->param('table') || '';
    my $options = '';
    
    if (!$DBH) {&connect_db($in) or return; }
    my $query = "SELECT * FROM $table LIMIT 1";
    my $sth = &exec_query($query) or return;
    for (my $i = 0; $i < $sth->{NUM_OF_FIELDS}; $i++) {
        $options .= qq~<OPTION value="$sth->{NAME}->[$i]">$sth->{NAME}->[$i]</OPTION>\n~;
    }
    $sth->finish();
    return ($options, $sth->{NUM_OF_FIELDS});
}

#=================================================#
#                  Mysqldump                      #
#=================================================#

sub mysqldump{
# ---------------------------------------------------
# Create dump file.  The result is either printed to the 
# screen or saved to a file.

    my ($in) = @_;

    my $file            = $in->param('dump_file')   || '';

    my $dump_whole_db   = $in->param('dump_whole_db')   || '';
    my $op_create_st    = $in->param('op_create_st')    || '';
    my $op_dump_db      = $in->param('op_dump_db')      || '';
    my $add_drop_table  = $in->param('add_drop_table')  || '';

    my $complete_insert = $in->param('complete_insert') || '';
    my $delayed_insert  = $in->param('delayed_insert')  || '';
    my $extended_insert = $in->param('extended_insert') || '';
    

    my $db_host         = $in->cookie($CONFIG{'db_host_cookie_name'}) || 'localhost';
    if ($CONFIG{'direct_connect'}) { $db_host = $CONFIG{'direct_host'} || $db_host; }

    my (@table_list, $sth, $query, $table, $table_selected, %ins_options, @ary);

    %ins_options = ();
    if ($complete_insert) {$ins_options{'complete_insert'} = 1;}
    if ($delayed_insert)  {$ins_options{'delayed_insert'} = 1;}
    if ($extended_insert) {$ins_options{'extended_insert'} =1;}


    if (!$DBH) {&connect_db($in) or return; }

    $query = "SHOW TABLES";
    $sth = &exec_query($query) or return;

    if ($dump_whole_db) { # store all tables in @table_list
        while ( ($table) = $sth->fetchrow_array() ) { 
            push (@table_list, $table); 
        }
    }
    else { # store selected tables in @table_list.
        while ( ($table) = $sth->fetchrow_array() ) { 
            if ($in->param("*dump_table*_$table")) { 
                push (@table_list, $in->param("*dump_table*_$table")); 
            }
        }
    }
    $sth->finish();

    if ($in->param("dump_to_screen")) {

        print "<HTML></HTML><HEAD></HEAD><BODY><pre>";

        print qq|# MySQL dump
# Generated by MySQLMan $VERSION (http://gossamer-threads.com/scripts/)
# Host: $db_host    Database: | . &get_db($in->param('data_source')) . qq| 
#--------------------------------------------------------

|;
        foreach $table_selected (@table_list) {

            # Write CREATE TABLE statement.
            if ($op_create_st) {
                print "#\n# Table structure for table '$table_selected'\n#\n";
                if ($add_drop_table) { print "DROP TABLE IF EXISTS $table_selected;\n"; }
                print &construct_create_table_statement($table_selected);
            }
        
            # Write INSERT statements.
            if ($op_dump_db) {
                print "#\n# Dumping data for table '$table_selected'\n#\n\n";
                if ($CONFIG{'dump_in_pages'}) {
                    $query = "SELECT COUNT(*) FROM $table_selected";
                    $sth = &exec_query($query) or return;
                    my ($rows) = $sth->fetchrow_array;
                    my $pages = $rows/$CONFIG{'dump_page_length'};
                    $pages = int($pages) + 1;
                    for (my $j=1; $j <= $pages; $j++) {
                        print &data_dump($table_selected, $j, %ins_options);
                    }
                    $sth->finish();
                }
                else { 
                    print &data_dump($table_selected, '', %ins_options);
                }
                print "\n";
            }
        }
        print "# ----------- Dump ends -----------</pre></BODY>";

    }
    else{
        if (!$file) {&sqlerr('Please provide a file name for mysqldump.'); return;}

        # Check if dump file specified already existed.
        if (not &dump_file_check($file)) {return};

        # create and dump to file.
        open(DUMP_FILE, ">>$file") or &cgierr("Unable to create dump file '$file': $!");

        print DUMP_FILE qq|# MySQL dump
# Generated by MySQLMan $VERSION (http://gossamer-threads.com/scripts/)
# Host: $db_host    Database: | . &get_db($in->param('data_source')) . qq| 
#--------------------------------------------------------

|;
        foreach $table_selected (@table_list) {

            # Write CREATE TABLE statement.
            if ($op_create_st) {
                print DUMP_FILE "#\n# Table structure for table '$table_selected'\n#\n";
                if ($add_drop_table) { print DUMP_FILE "DROP TABLE IF EXISTS $table_selected;\n"; }
                print DUMP_FILE &construct_create_table_statement($table_selected);
            }
        
            # Write INSERT statements.
            if ($op_dump_db) {
                print DUMP_FILE "#\n# Dumping data for table '$table_selected'\n#\n\n";
                if ($CONFIG{'dump_in_pages'}) {
                    $query = "SELECT COUNT(*) FROM $table_selected";
                    $sth = &exec_query($query) or return;
                    my ($rows) = $sth->fetchrow_array;
                    my $pages = $rows/$CONFIG{'dump_page_length'};
                    $pages = int($pages) + 1;
                    for (my $j=1; $j <= $pages; $j++) {
                        print DUMP_FILE &data_dump($table_selected, $j, %ins_options);
                    }
                    $sth->finish();
                }
                else { 
                    print DUMP_FILE &data_dump($table_selected, '', %ins_options);
                }
                print DUMP_FILE "\n";
            }
        }
        print DUMP_FILE "# ----------- Dump file ends -----------";
        close(DUMP_FILE);

        &show_tables($in, "Dump file created");
    }
}

sub construct_create_table_statement{
# ---------------------------------------------------
# Given a table name, construct a a CREATE TABLE statement
# that creates the exact table.

    my($table) = @_;
    my ($sth, $query, $create_query, $col_spec, $type, $key_name, $non_unique, $not_null, @key_col, @build_key_statement, $key_spec);
    my ($field, $type_name, $length_set, $attributes, $null, $key, $default, $extra);

    my @col_spec = ();
    
    $query = qq~DESCRIBE $table~;
    $sth = &exec_query($query) or return;

    # Construct column specifications.
    while (my @ary = $sth->fetchrow_array) {
        ($field, $type_name, $length_set, $attributes, $null, $key, $default, $extra) = &parse_col_spec(@ary) or return;
        if ($null) { $not_null = '' }
        else { $not_null = 'NOT NULL' }
        if ($length_set) { $length_set = "($length_set)"}
        if ((defined $default) or ($not_null)) { 
            if (not defined $default) {$default = '';}
            $default = "DEFAULT " . $DBH->quote($default); 
        }
        $col_spec = "  $field $type_name$length_set $attributes $default $not_null $extra";
        push(@col_spec, $col_spec);
    }
    $sth->finish();

    # find out if there are keys in the table.
    $query = qq~SHOW KEYS FROM $table~;
    $sth = &exec_query($query) or return;

    $key_name = '';
    @build_key_statement = ();
    while (my @ary = $sth->fetchrow_array) {
        my $cur_non_unique = $ary[1];
        my $cur_key_name   = $ary[2];
        my $col            = $ary[4];
        if ($key_name eq $cur_key_name) { push(@key_col, $col); }
        else {
            if ($key_name) {
                if ($key_name eq 'PRIMARY') { $key_spec = '  PRIMARY KEY (' . join("," , @key_col) . ')'; }
                elsif ($non_unique)         { $key_spec = "  KEY $key_name (" . join("," , @key_col) . ')'; }
                else                        { $key_spec = "  UNIQUE $key_name (" . join("," , @key_col) . ')'; }
                push(@build_key_statement, $key_spec);
            }
            @key_col = ();
            push(@key_col, $col);
            $key_name   = $cur_key_name; 
            $non_unique = $cur_non_unique;
        }
    }

    $sth->finish();

    if ($key_name) {
        if ($key_name eq 'PRIMARY') { $key_spec = '  PRIMARY KEY (' . join("," , @key_col) . ')'; }
        elsif ($non_unique)         { $key_spec = "  KEY $key_name (" . join("," , @key_col) . ')'; }
        else                        { $key_spec = "  UNIQUE $key_name (" . join("," , @key_col) . ')'; }
        push(@build_key_statement, $key_spec);
    }
    
    if (@build_key_statement) {$create_query = "CREATE TABLE $table (\n" . join (",\n", @col_spec) . ",\n" . join (",\n",@build_key_statement) . "\n);\n\n";}
    else {$create_query = "CREATE TABLE $table (\n" . join (",\n", @col_spec) . "\n);\n\n";}

    return $create_query;
}

sub data_dump{
# ---------------------------------------------------
# Construct INSERT TABLE statements for a given table.

    my ($table, $page, %options) = @_;
    my ($sth, $query, $count, @ary, $field, @records, @fields, $insert_statement, $not_first_rec);

    my $complete_insert = $options{'complete_insert'} || '';
    my $delayed_insert  = $options{'delayed_insert'}  || '';
    my $extended_insert = $options{'extended_insert'} || '';

    $query = qq~SELECT COUNT(*) FROM $table~;
    $sth = &exec_query($query) or return;
    ($count) = $sth->fetchrow_array;
    $sth->finish();

    if (!$page) { $query = "SELECT * FROM $table"; }
    else{
        my $start_row = $CONFIG{'dump_page_length'} * ($page - 1);
        $query = "SELECT * FROM $table LIMIT $start_row, $CONFIG{dump_page_length}"; 
    }
    $sth = &exec_query($query) or return;

    # if complete-insert option is selected, construct the fields in the table. 
    my $complete_op_fields = '';
    my @field_list = ();
    if ($complete_insert) {
        for (my $i = 0; $i < $sth->{NUM_OF_FIELDS}; $i++) {
            push (@field_list, $sth->{NAME}->[$i]);
        }
        $complete_op_fields = '(' . join ("," , @field_list) . ')';
    }

    # if delayed-insert option is selected, added DELAYED keyword in INSERT statements.
    my $delayed_keyword = '';
    if ($delayed_insert) { $delayed_keyword = 'DELAYED'}
    
    $not_first_rec = 0;
    while (@ary = $sth->fetchrow_array) {
        @fields = ();
        foreach $field (@ary) {
            if (defined($field)) { push (@fields, $DBH->quote($field)); }
            else {push (@fields, 'NULL')}
        }
        
        if ($extended_insert) {
            if ($not_first_rec) {
                $insert_statement .= ",(" . join (",", @fields) . ")"
            }
            else {
                $insert_statement = "INSERT $delayed_keyword INTO $table $complete_op_fields VALUES(" . join (",", @fields) . ")";
                $not_first_rec = 1;
            }
        }
        else{
            $insert_statement = "INSERT $delayed_keyword INTO $table $complete_op_fields VALUES(" . join (",", @fields) . ");\n";
        }
        
        if (!$extended_insert) { push (@records, $insert_statement); }
    }
    if ($extended_insert) { 
        if ($insert_statement) {$insert_statement .= ";\n";}
        push (@records, $insert_statement); 
    }

    $sth->finish();
    return @records;
}

sub dump_file_check{
# ---------------------------------------------------
# Check to see if the dump file specified already existed.
# returns 1 if there isn't such file; undef if there is.

    my $file = shift;
    my ($dir)= $file =~ m,(.*)[/\\][^/\\]+,;
    if (-e $file) {
        &sqlerr("File '$file' already exists.  Please use another file name.");
        return;
    }
    elsif ($dir and ! -w $dir) {
        &sqlerr("Can not create file in '$dir' directory. Please use another file name.");
        return;
    }
    elsif (!$dir and ! -w '.') {
        &sqlerr("Can not create file in '.' directory. Please use another file name.");
        return;
    }
    else {
        return 1;
    }
}

sub get_checkbox_tables_in_database{
# ---------------------------------------------------
# get the list of tables of the database that the user 
# is currently in and create a checkbox for each name for 
# SQL dump

    my ($in) = @_;
    my ($sth, $query, $i, $table, $table_checkboxes);

    if (!$DBH) {&connect_db($in) or return; }

    $query = "SHOW TABLES";
    $sth = &exec_query($query) or return;
    
    $i = 0;
    $table_checkboxes = '';
    while (($table) = $sth->fetchrow_array) {

        $table_checkboxes .= qq~\n<input type="checkbox" name="*dump_table*_$table" value="$table">$table<BR>~;
        
        # output the check boxes such that there are 5 rows in a column.
        if ( ($i+1)%5 == 0 ) { $table_checkboxes .= "</TD><TD VALIGN = top>"; }
        $i++;
    }   
    $sth->finish();
    return "<TABLE><TR><TD>$table_checkboxes</TD></TR></TABLE>";
}

#=================================================#
#                  UTILITIES                      #
#=================================================#

sub build_saved_query_table{
# ---------------------------------------------------
# build a table for queries saved.

	my $in = shift;
	my $query_table = '';
	
	if ($in->cookie('query_1')) {
		$query_table = qq~ 
			<TABLE border=1 cellspacing=0 cellpadding=5>		
			<FORM METHOD="POST" ACTION="$CONFIG{'script_url'}">
			<INPUT TYPE="hidden" NAME=do VALUE='sql_monitor'>
			<INPUT TYPE="hidden" NAME=data_source VALUE="~ . $in->param('data_source') . qq~">
			<INPUT TYPE="hidden" NAME=action VALUE="monitor">
			<INPUT TYPE="hidden" NAME=from_monitor VALUE="1">
			<TR>
			<TH><INPUT TYPE="submit" value="Run Query" name = "from_saved_queries"></TH>
			<TH>SAVED QUERY</TH>
			<TH>COMPLETE QUERY</TH>
			<TH><INPUT TYPE="submit" value="Delete Query" name = "delete_query"></TH>
			</TR>
		~;
				
		for (my $j=10; $j >= 1; $j--) {
			
			my $partial_query;
			my @temp = split / /, $in->cookie("query_$j");

			if ($#temp > 10) {
				$partial_query = "$temp[0] $temp[1] $temp[2] ... $temp[$#temp-2] $temp[$#temp-2] $temp[$#temp]"
			}
			else {
				$partial_query = $in->cookie("query_$j");
			}

			if ($in->cookie("query_$j")) {
				$query_table .= 
				qq~<TR>
				<TD><CENTER><INPUT TYPE="radio" NAME="query_select" VALUE="$j"></CENTER></TD>
				<TD>~ . $partial_query . qq~</TD>
				<TD><CENTER><a href="$CONFIG{'script_url'}?do=show_query&query_displayed=$j" target="_">Show</a><CENTER></TD>
				<TD><CENTER><INPUT TYPE="checkbox" NAME="delete_query_$j" VALUE="1"></CENTER></TD>
				</TR>\n~;
			}
		}

		$query_table .= qq~</FORM></TABLE>~;
	}

	return $query_table;
}

sub create_temp_file{
# ---------------------------------------------------
# creates a temp file and return the path to the file.

    my ($in) = @_;
    my $temp_file = &get_temp_file_name();
    my $fh = $in->param('upload_local_file') || '';
    
    if (!$fh) { &sqlerr('Query file not specified.'); return;}

    if (!open(OUTFILE, ">$temp_file")) {
      &cgierr("There was an error opening temp file '$temp_file' for writing.\n");
    }

    open (OUTFILE,">>$temp_file");
    my ($buffer);
    while (read($fh,$buffer,1024)) {
        print OUTFILE $buffer;
    }

    close($fh);
    close(OUTFILE);
    chmod (0666, "$temp_file");

    return $temp_file;
}

sub get_temp_file_name {
# ---------------------------------------------------
# This function creates a random temp file name, looks
# up the temp directory and returns the path to the 
# temp file.

    my $temp_file_path = '';    
    my $directory      = '.';
    my @temp_dirs =("/usr/tmp","/var/tmp","C:/temp","/tmp","/temp","/WWW_ROOT");
    foreach (@temp_dirs) {
        if (-d $_ && -w _) { $directory = $_; last; }
    }    
    my $rand_name = "GTMM" . time . $$ . int(rand(1000));
    while (-e $directory . '/' . $rand_name) {
        $rand_name = "GTMM" . time . $$ . int(rand(1000));
    }
    $temp_file_path = "$directory/$rand_name";
    return $temp_file_path;
}

sub parse_col_spec{
# ---------------------------------------------------
# Given a column specification from DESCRIBE TABLE, the function
# finds out 
#    field name
#    typename 
#    length/set
#    attributes
#    if nullable
#    if set as key
#    default
#    extra property.

    my @ary = @_;

    my $field   = $ary[0]; 
    my $type    = $ary[1];
    my $null    = $ary[2];
    my $key     = $ary[3];
    my $default = $ary[4];
    my $extra   = $ary[5];

    my ($chop_att, $attributes, $dump, $value_q, $value_unquote, $type_name, $length_set, @type_field, @length_set, @new_length_set, $new_length_set);

# Get column type.
    ($type_name, $dump) = split /([(])/, $type, 2;
    
# Get length/set
    $length_set = '';

    my @t   = split / /, $type;
    if ( ($t[0] ne $t[$#t]) && ( ($type_name ne 'set') && ($type_name ne 'enum') )) { for (my $i=0; $i < $#t; $i++) { $chop_att .= $t[$i] } }
    else {$chop_att = $type}
    my @tmp = split /([()])/, $chop_att;
    my $flag = 0;
    for (my $i=0; $i < $#tmp; $i++) {
        if ($flag) { $length_set .= $tmp[$i]; }
        if ($tmp[$i] eq '(') { $flag = 1;}
    }

# Get Attribute.
    @type_field = split / /, $type;
    if ( ($type_field[0] ne $type_field[$#type_field]) && ( ($type_name ne 'set') && ($type_name ne 'enum') )) { $attributes = $type_field[$#type_field] }
    else { $attributes = ''; }

    if ($type_name eq 'set' || $type_name eq 'enum') {

# Get the elements in length/set.
        @length_set = &parse_length_set($length_set);

        foreach  (@length_set) {
            if ($_ ne "''") { 
                ($value_unquote) = $_ =~ m{^\'(([^\']|\'\')+)\'};
                $value_unquote =~ s/''/\\'/g;
                $value_unquote =~ s/,/\\,/g;
                $value_q = "'" . $value_unquote . "'";
            }
            push(@new_length_set, $value_q);
        }
        $new_length_set = join ",", @new_length_set;
    }
    else {$new_length_set = $length_set}
    
    return ($field, $type_name, $new_length_set, $attributes, $null, $key, $default, $extra);
}

sub set_col_not_null{
# ---------------------------------------------------
# Takes in a table name and the column name.  Then 
# function will make the column specified not nullable.
# It will first read in the current spec of the column 
# and then reconstruct the spec to set the column not null.

    my $in = shift;
    my $table = $in->param('table') || '';
    my $col = $in->param('col')     || '';

    my ($sth, $query, @attr, $new_spec, $col_q);

    $col_q = $DBH->quote($col);

    if (!$DBH) {&connect_db($in) or return; }
    $query = "SHOW COLUMNS FROM $table LIKE $col_q";
    $sth = &exec_query($query) or return;
    @attr = $sth->fetchrow_array();
    
    # reconstruct spec and set column not null.
    for (my $i=0; $i < $sth->{NUM_OF_FIELDS}; $i++) {
        if    ($i == 2)                             {$new_spec .= ' NOT NULL ';}
        elsif ( ($i == 4) && (defined($attr[$i])) ) {$new_spec .= " DEFAULT " . $DBH->quote($attr[$i]);}
        elsif (($i == 3) || ($i == 6))              {}
        else                                        {$new_spec .= " $attr[$i] ";}
    }
    $sth->finish();

    $query = "ALTER TABLE $table CHANGE $col $new_spec";
    $sth = &exec_query($query) or return;
    $sth->finish();
}

sub get_pri_key{
# ---------------------------------------------------
# Gets the primary key of the table specified.

    my $table = shift;
    my ($sth, $query, @ary, @pri_key);
    
    @pri_key = ();

    $query = "DESCRIBE $table";
    $sth = &exec_query($query) or return;
    while (@ary = $sth->fetchrow_array) {
        if ($ary[3] eq 'PRI') { push(@pri_key, $ary[0]); }
    }
    $sth->finish();

    return @pri_key;
}

sub get_table_list{
# ---------------------------------------------------
# Gets the list of tables which the query input is querying 
# from.  Since the function is only used in "sub table_browse";
# 

    my $query = shift;
    my (@query, $token, $flag, @table_list, $cur_token, $pre_token, $stop, $cmd, $explain_select, $got_list);

    #strip beginning and endding space.
    $query =~ s/^\s+//;  
    $query =~ s/\s+$//;
    $query =~ s/(\r|\n)+/ /g;

    @query = split /([ ,])/, $query;
    $cmd = lc($query[0]);

    # find table list from:
    # 1. describe / desc
    # 2. explain table 
    if (($cmd eq 'describe') || ($cmd eq 'desc') || ($cmd eq 'explain')) {

        @table_list = ();
        $flag = 0;
        
        for (my $i=1; $i<=$#query;$i++){
            if (defined($query[$i])){
                if (($query[$i] ne '') && ($query[$i] ne ' ') && ($query[$i] ne ',') ){
                    
                    # get the first name after the command.
                    if ($flag == 0) {
                        push (@table_list, $query[$i]);
                        my $tmp = lc($query[$i]);
                        if ($tmp eq 'select') { $explain_select = 1 }
                        $flag = 1;
                    }
                }
            }
        }
        $got_list = 1;
    }

    # find table list from queries like:
    # 1. select queries
    # 2. show queries
    # 3. explain select ....
    if (!$got_list || $explain_select) {
        
        @table_list = ();
        $flag = 0;
        $stop = 0;
        $pre_token = '';

        foreach (@query) {
            if (($_ ne '') && ($_ ne ' ')) {
                $token = lc($_);
                if ($flag == 1) { 

                    # determine the type of the current token.
                    if ($token ne ',') { $cur_token = 'word' }
                    else { $cur_token = 'comma' }

                    # stop then the "from" clause ends
                    if (($cur_token eq 'word') && ($cur_token eq $pre_token) ) { $stop = 1; }
                    
                    if (!$stop && $token ne ',') { push (@table_list, $_) }
                    $pre_token = $cur_token;
                }   
                if ($token eq 'from') { $flag = 1; }
            }
        }
    }
    return @table_list;
}

sub form_fields{
# ---------------------------------------------------
# Create a form input table for insert and edit.  $update
# is a flag indicating whether or not it is from edit.  @value 
# consists the list of original values (in order) in the 
# record being updated.  Note that in order to set a field
# to be null, the input value has to be null.  In other words, 
# if there is value in input field and null checkbox is checked, 
# the null option will be overwritten and the value in the input 
# field will be taken.
#
# Option: you can set it in the config file such that any colume 
# with type TIMESTAMP will not be shown in the form.

    my($in, $update, @value) = @_;
    
    my $table = $in->param('table') || '';
    my ($query, $sth, $form_table, @ary, @type, @domain, @domain_new, %domain_h, $value_unquote,
        $flag, $double_comma, $length_set);

	my @timestamp_hidden = ();

    if (!$DBH) {&connect_db($in) or return; }
    
    $query = "DESCRIBE $table";
    $sth = &exec_query($query) or return; 

    $form_table = "<TABLE BORDER = 2><TR><TH>Fields</TH><TH>Type</TH><TH>Function</TH><TH>Value</TH>";
    if ($CONFIG{'insert_null'}) { $form_table .= "<TH>Null</TH></TR>"; }
    else {$form_table .= "</TR>";}

    my $k = 0; # Value counter.  Used to identify which element in @value is considered.

    while (@ary = $sth->fetchrow_array) {
    my $tabindex_count = $k+1;

        # $ary[1] is in the form "type_name(length_set) attribute"
        # 
        @type = split /([(])/, $ary[1];
        $length_set = '';
        for (my $j=2; $j <= $#type; $j++) {$length_set .= $type[$j];}
        chop $length_set;
        
    
        # Handle type 'Enum'        
        if ($type[0] eq 'enum') {
            $form_table .=  qq~\n<TR><TD>$ary[0]</TD><TD>enum</TD><TD>~ . &function_select($ary[0]) . qq~</TD><TD><select name="*insert*_$ary[0]" tabindex="$tabindex_count">~;

            if (defined($value[$k]) && $update) { 
                $form_table .= qq~\n<option>$value[$k]</option>~; 
            }

            # Create an empty choice.  This choice available even if it is not specified in 
            # enum.  We need it in case the value needs to be null.
            $form_table .= qq~\n<option></option>~;
        
            
            # All other choices in specified in the enumeration.
            @type = &parse_length_set($length_set);

            for (my $i=0; $i <= $#type; $i++) {
                if (defined($type[$i]) && $type[$i] ne "''" && $type[$i] ne '') {
                    ($value_unquote) = $type[$i] =~ m{^\'(([^\']|\'\')+)\'};
                    # Since MySQL stores single quotes in 2 single quotes in column specs,
                    # we translate them back to 1. 
                    $value_unquote =~ s/''/'/g;
                    $value_unquote =~ s/\\\\/\\/g;
                    $form_table .= qq~\n<option>$value_unquote</option>~;
                }
            }
            # Null checkbox
            if ($CONFIG{'insert_null'}) {
                if ($ary[2] eq 'YES') { 
                    if ( !defined($value[$k]) && $update) { $form_table .= qq~</select></TD><TD><input type="checkbox" name="*insert*_$ary[0]_null" value="NULL" checked></TD></TR>~; }
                    else {$form_table .= qq~\n</select></TD><TD><input type="checkbox" name="*insert*_$ary[0]_null" value="NULL"></TD></TR>~; }
                }
                else { $form_table .= qq~\n</select></TD><TD></TD></TR>~; }
            }
        }

        # Handle type 'Set'
        elsif($type[0] eq 'set'){
        
            my $j = 0;

            $form_table .= qq~\n<TR><TD>$ary[0]</TD><TD>set</TD><TD>~ . &function_select($ary[0]) . qq~</TD><TD>~;
            
            # For update, check if '' is in the set.
            @domain = split /(,)/, $value[$k];
            foreach (@domain) { 
                if ($_ ne ',') { 
                    if ($_ ne '') {push (@domain_new, $_) }
                    else {push (@domain_new, "''")}
                }
            }
            if ($domain[$#domain] eq ',') { push (@domain_new, "''"); }
            %domain_h = map{$_ => 1} @domain_new;

            @type = &parse_length_set($length_set);         
            
            for (my $i=0; $i <= $#type; $i++) {
                
                if (defined($type[$i]) && $type[$i] ne '') {
                    
                    if ($type[$i] ne "''") { ($value_unquote) = $type[$i] =~ m{^\'(([^\']|\'\')+)\'}; }
                    else { $value_unquote = $type[$i]; }
                    
                    if ($value_unquote ne "''") { $value_unquote =~ s/''/'/g; }
                    $value_unquote =~ s/\\\\/\\/g;
                    
                    # Create checkboxes for each element in the set.  Checkboxes are checked if 
                    # value is selected in the original set.
                    if ($domain_h{$value_unquote}){
                        $form_table .= qq~\n<input type="checkbox" name="*insert*_$ary[0]_set_$j" value="~ . $in->html_escape($value_unquote) . qq~" checked tabindex="$tabindex_count">$value_unquote<BR>~;
                        $j++;
                    }
                    else {
                        $form_table .= qq~\n<input type="checkbox" name="*insert*_$ary[0]_set_$j" value="~ . $in->html_escape($value_unquote) . qq~" tabindex="$tabindex_count">$value_unquote<BR>~;
                        $j++;
                    }
                }
            }
            if ($CONFIG{'insert_null'}) {
                if ($ary[2] eq 'YES') { 
                    if ( !defined($value[$k]) && $update) {$form_table .= qq~</TD><TD><input type="checkbox" name="*insert*_$ary[0]_null" value="NULL" checked></TD></TR>~; }
                    else{$form_table .= qq~\n</TD><TD><input type="checkbox" name="*insert*_$ary[0]_null" value="NULL"></TD></TR>~;}
                }
                else { $form_table .= qq~</TD><TD></TD></TR>~; }
            }
        }

        # Handle all other types
        else {
			my $type_lookup   = lc($ary[1]);
			my $hidden_field  = '';

			# hide columns with type TIMESTAMP if necessary.
			if (($type_lookup =~ m/timestamp/) && !$CONFIG{'show_timestamp_field'}) {  
				if (!$update) {
					$hidden_field = qq~<INPUT TYPE="hidden" NAME="*insert*_$ary[0]_null" VALUE="NULL">~;
					push (@timestamp_hidden, $hidden_field);
				}
				else{
					$hidden_field = qq~<INPUT TYPE="hidden" NAME="*insert*_$ary[0]" VALUE="~ . $in->html_escape($value[$k]) . qq~">~;
					push (@timestamp_hidden, $hidden_field);
				}
			}
			else{
				# handle fields with type TEXT or BLOB.  
				if ($type_lookup =~ m/text/ || $type_lookup =~ m/blob/) {
					$form_table .= qq~\n<TR><TD>$ary[0]</TD><TD>$ary[1]</TD><TD>~ . &function_select($ary[0]) . qq~</TD><TD><TEXTAREA NAME="*insert*_$ary[0]" ROWS="5" COLs="30" WRAP="VIRTUAL" tabindex="$tabindex_count">~ . $in->html_escape($value[$k]) . qq~</TEXTAREA></TD>~; 
				}
				else{
		            $form_table .= qq~\n<TR><TD>$ary[0]</TD><TD>$ary[1]</TD><TD>~ . &function_select($ary[0]) . qq~</TD><TD><INPUT TYPE="text" NAME="*insert*_$ary[0]" VALUE="~ . $in->html_escape($value[$k]) . qq~" SIZE="30" tabindex="$tabindex_count"></TD>~; 
				}
	            if ($CONFIG{'insert_null'}) {
	                if ($ary[2] eq 'YES') { 
	                    if ( !defined($value[$k]) && $update) {$form_table .= qq~<TD><input type="checkbox" name="*insert*_$ary[0]_null" value="NULL" checked></TD></TR>~; }
	                    else { $form_table .= qq~<TD><input type="checkbox" name="*insert*_$ary[0]_null" value="NULL"></TD></TR>~; }
	                }
	                else { $form_table .= qq~<TD></TD></TR>~; }
	            }
			}
        }
        $k++;
    }
    $form_table .= "</TABLE>";
    $sth->finish();
    
	if (@timestamp_hidden) {
		$form_table .= join("", @timestamp_hidden);
	}

    return $form_table;
}


sub parse_length_set{
# ---------------------------------------------------
# A simple parser that gets each element in the length set field. 
# Basically, it counts the number for sigle quotes.  Each element 
# must have even number of quotes.  If a comma encountered, then 
# the check the number for quotes seen so far.  If the number is odd, 
# then concate the comma to temp string, otherwise, push the current 
# tmep string to array.

    my ($length_set) = @_;

    my @type = split /([,'])/, $length_set;  # list of quoted elements in length_set
    my @new_type = ();
    my $remainder = 0;
    my $cur_choice = '';
    my $q_count = 0;
    my $begin = 0;
    my $k = 0;

    foreach my $element (@type) {
        if ($k == $#type) { 
            $cur_choice .= $element;
            push(@new_type, $cur_choice); 
        }
        elsif ($element ne '') {            
            if ($element eq ',') {
                $remainder = $q_count%2;
                if ($remainder) {
                    if ($begin) {
                        push(@new_type, $cur_choice);
                        $begin = 0;
                        $q_count = 0;
                        $cur_choice = '';
                    }
                    else{
                        $begin = 1;
                        $cur_choice .= $element;
                        $q_count = 0;
                    }
                }
                else{
                    if ($begin) {
                        $cur_choice .= $element;
                    }
                    else {
                        push(@new_type, $cur_choice);
                        $q_count = 0;
                        $cur_choice = '';
                    }
                }   
            }
            elsif($element eq "'"){
                $q_count++;
                $cur_choice .= $element;
            }
            else{
                $cur_choice .= $element;
            }
        }
        $k++;
    }
    return @new_type;
}


sub compose_new_condition{
# ---------------------------------------------------
# Reconstructs the input from "sub form_fields" to an array
# of "field = value" pairs.
#
# The functions is modified after v1.03
# The way that the script inserts a record is changed to
#
# INSERT INTO tablename (column1,column2,...columnn)
# VALUES (value1,value2, .. valuen)
#
# as the SET col=val isn't supported by all versions of mysql.
# Therefore, if is_insert flag in on, the function returns
# a array in the following format:
#
# (field_name_1, value_for_field_name_1, field_name_2, value_for_field_name_2, ...)
#
# Please note that the value in $value is quoted.
#

    my ($in, $is_insert) = @_;
    my $table = $in->param('table') || '';
    my ($query, $prep, $sth, @ary, $value, @insert_fields, @type, @set, $new_record, $value_unquote);


    $query = "DESCRIBE $table";
    $prep = &exec_query($query) or return;

    while (@ary = $prep->fetchrow_array) {
        
        @type = split /[(),]/, $ary[1];
    
        # Handle columns of type 'SET'
        if ($type[0] eq 'set') {            
            my $j = 0;  # checkbox counter
            my $k = 0;  # counter for how many checkboxes are checked.
            
            # check each checkbox box to see if they are check.
            for (my $i=1; $i < ($#type + 1); $i++) {
                ($value_unquote) = $type[$i] =~ m{^\'(([^\']|\'\')+)\'};                
                if (defined($type[$i])) {
                    if ( defined($in->param("*insert*_$ary[0]_set_$j")) ) {
                        if ( $in->param("*insert*_$ary[0]_set_$j") ne "''" ) { push (@set, $in->param("*insert*_$ary[0]_set_$j") ); }
                        else { push (@set, ''); }
                        $k++;
                    }
                $j++;
                }
            }

            $value = $DBH->quote( join(",", @set) );
            
            # If none of the checkboxes is checked, check for null option. 
            if (!$k) {
                if ($in->param("*insert*_$ary[0]_null")) { $value = 'NULL'; }
                else { $value = '""'; }
            }

        }
        # Handle all other types.
        else {
            # if nothing in input field.
            if (!$in->param("*insert*_$ary[0]") && ($in->param("*insert*_$ary[0]") ne '0')) {
                
                # check for null
                if ($in->param("*insert*_$ary[0]_null")) { $value = 'NULL'; }               
                
                # check for function.
                else { 
                    if ( $in->param("*insert*_$ary[0]_function") ) {$value = $in->param("*insert*_$ary[0]_function") . '()'; }  
                    else { $value = '""'; }
                }
            }

            # check if any function is needed to apply on the input.
            elsif ($in->param("*insert*_$ary[0]_function")){ 
                $value = $in->param("*insert*_$ary[0]_function") . '(' . $DBH->quote($in->param("*insert*_$ary[0]")) .')'; 
            }

            # otherwise make the field equal to the value entered.
            else{ $value = $DBH->quote($in->param("*insert*_$ary[0]")); }
        }
		if ($is_insert) {
			push (@insert_fields, $ary[0]);
			push (@insert_fields, $value);
		}
		else{
	        push (@insert_fields, "$ary[0] = $value");  
		}
    }
    $prep->finish();

    return (@insert_fields);
}

sub function_select {
# ---------------------------------------------------
# Creates enumeration of functions available in select input.

    my $field = shift;
    return GT::Template->parse ('functions.txt', { field => $field}, { root => $CONFIG{template_dir} })
}

sub get_key_table{
# ---------------------------------------------------
# Creates a key/index table.  A "drop" link is created 
# together with each each read in.

    my $in = shift;
    my $data_source = $in->param('data_source') || '';
    my $table = $in->param('table') || '';
    my ($sth, $query, @ary, $unique, $non_unique, $key_name, $column_name, $keys, $key_table);
    
    $query = "SHOW INDEX from $table";
    $sth = &exec_query($query) or return;
    while (@ary = $sth->fetchrow_array) {
        my $non_unique  = $ary[1];
        my $key_name    = $ary[2];
        my $column_name = $ary[4];
        if ($non_unique) {$unique = 'NO';}
        else {$unique = 'YES';}
        $keys .= qq~<TR><TD>$key_name</TD><TD>$unique</TD><TD>$column_name</TD>
                   <TD><A href="$CONFIG{'script_url'}?do=alter_table&data_source=$data_source&table=$table&action=drop_key&key_name=$key_name">Drop</A></TD></TR>~;
    }
    $sth->finish();
    
    if ($keys) { $key_table = "<TABLE BORDER=2><TR><TH>Key name</TH><TH>Unique</TH><TH>Field</TH><TH>Action</TH></TR>$keys</TABLE>"; }
    else { $key_table = ''; }
    return $key_table;
}

sub get_db{
# ---------------------------------------------------
# Gets the database name from the data source which is 
# in the format "DBI:mysql:database_name:host"

    my $db = shift;

    my @dsn = split /([:])/, $db;
    $db = $dsn[4]; 
    if ($CONFIG{'direct_connect'}) { $db = $CONFIG{'direct_db'}; }

    return $db;
}

sub get_cols{
# ---------------------------------------------------
# Takes in a table name and return the columns in an ]
# array.

    my ($in, $table) = @_;
    my ($query, $sth, @cols);
    if (!$DBH) {&connect_db($in) or return; }
    $query = "SELECT * FROM $table LIMIT 1";
    $sth = &exec_query($query) or return;
    for (my $i = 0; $i < $sth->{NUM_OF_FIELDS}; $i++) {
        push (@cols, $sth->{NAME}->[$i]);
    }
    $sth->finish();
    return @cols;
}


sub valid_name_check{
# ---------------------------------------------------
# Checks to see if the input database/table name is a 
# valid one.  The function checks the following:
# 1. if a name is entered at all;
# 2. if there are spaces in the name;
# 3. if the name is consisted of valid characters; and
# 4. if the name is consisted of only numbers.

    my $name = shift;

    $name =~ s/^\s+//;  
    $name =~ s/\s+$//;

    my @name = split / /, $name;

    if (!$name)                 { &sqlerr("Please provide a valid name."); }
    elsif ($#name > 0)          { &sqlerr("Spaces are not allowed in name."); }
    elsif ($name =~ m/[^\w_\$]/){ &sqlerr("Invalid name.  A name may consist of characters, numbers, and also '_' and '\$'."); }
    elsif (!($name =~ m/\D/))   { &sqlerr("Invalid name.  A name may not consist only of numbers."); }
    else  {return 1;}
}

sub alias_name_check{
# ---------------------------------------------------
# Checks to see if the input table name is of the format
# 1. database.table.column  OR
# 2. table.column
    
    my ($table, $in) = @_;
    my @alias_table = split /[.]/, $table;

    if ($#alias_table > 0) { &html_demo_prompt($in, 'Action not allowed in demo mode'); }
    else {return 1;}
}

sub concate_col_spec{
# ---------------------------------------------------
# Reconstruct the input variables into a string in the form 
# "field_name(type(length_set) attribute DEFAULT default_value extra)"

    my ($in, $i) = @_;
    my $col_spec;
    
    $col_spec = '';
    $col_spec .= $in->param("field_$i") . ' ';
    $col_spec .= $in->param("type_$i");
    if ( $in->param("length_set_$i") ) { $col_spec .= '(' . $in->param("length_set_$i") . ')'; }
    $col_spec .= ' ' . $in->param("attributes_$i") . ' ';
    $col_spec .= $in->param("null_$i") . ' ';
    if ( $in->param("default_$i") ) { $col_spec .= 'DEFAULT ' . $DBH->quote($in->param("default_$i")) . ' '};
    $col_spec .= $in->param("extra_$i");
    
    return $col_spec;
}


sub link_page {
# ---------------------------------------------------
# Provides hyperlinks to next, previous, or top pages when needed
    
    my ($in, $rows, $table, $fields, $example, $index, $total_rec_num) = @_;
    my ($data_source, $cur_page, $page, $output, $sort_index, $action, $where, $query, $cur_rec_num, $more_page);
    
    $data_source = $in->param("data_source") || '';
    $cur_page    = $in->param("page")        || 1;
    $sort_index  = $index;
    $action      = $in->param("browse_action") || $in->param("action") || '';
    $where       = $in->escape($in->param("where")) || '';
    $query       = $in->escape($in->param("query")) || '';
    
    $example = $in->escape($example);

    $cur_rec_num = ($cur_page - 1) * $CONFIG{'page_length'} + $rows;
    if ($cur_rec_num < $total_rec_num) { $more_page = 1; }
    else { $more_page = 0; }


    $output = '';

    # the very first page.
    if ( ($cur_page == 1) and ($rows == $CONFIG{'page_length'}) and ($more_page)) {
        $page = $cur_page + 1; 
        $output .= qq~< <A href="$CONFIG{'script_url'}?do=browse&data_source=$data_source&table=$table&page=$page&sort_index=$sort_index&action=$action&fields=$fields&where=$where&example=$example&query=$query">Next page</A> >~;
    }

    # the very last page.
    elsif ((!$more_page) and ($cur_page != 1)){ 
        $page = $cur_page - 1;
        $output .= qq~< <A href="$CONFIG{'script_url'}?do=browse&data_source=$data_source&table=$table&page=$page&sort_index=$sort_index&action=$action&fields=$fields&where=$where&example=$example&query=$query">Previous page</A> > ~;
    } 

    # any page between the first and the last page.
    elsif ( ($cur_page != 1) and ($rows == $CONFIG{'page_length'}) and ($more_page)){ 
        $page = $cur_page + 1;
        $output .= qq~< <A href="$CONFIG{'script_url'}?do=browse&data_source=$data_source&table=$table&page=$page&sort_index=$sort_index&action=$action&fields=$fields&where=$where&example=$example&query=$query">Next page</A> > ~;
        $page = $cur_page - 1;
        $output .= qq~< <A href="$CONFIG{'script_url'}?do=browse&data_source=$data_source&table=$table&page=$page&sort_index=$sort_index&action=$action&fields=$fields&where=$where&example=$example&query=$query">Previous page</A> > ~;
    }
    # else there is only one page to display.  As a result, no links are available.

    # link to jump back to the first page.
    if (($cur_page != 1)) { 
        $output .= qq~< <A href="$CONFIG{'script_url'}?do=browse&data_source=$data_source&table=$table&page=1&sort_index=$sort_index&action=$action&fields=$fields&where=$where&example=$example&query=$query">Top page</A> >~;
    }
    return $output;
}

sub link_page_jump{
# ---------------------------------------------------
# Produces a text field to let the user enter a number 
# and the user will be brought to the page specified.
    
    my($sth, $table, $where, $action, $query, $data_source, $pages, $output);

    my ($in, $fields, $example, $index, $rows) = @_;

    $data_source = $in->param("data_source") || '';
    $table       = $in->param("table")       || '';
    $where       = $in->param("where")       || '';
    $action      = $in->param("browse_action") || $in->param("action") || ''; 
    
    $pages = $rows / $CONFIG{'page_length'};
    if (($rows % $CONFIG{'page_length'}) != 0) { $pages = int($pages) + 1 }
    if ($rows > $CONFIG{'page_length'}) {
        return &html_page_jump($in, $pages, $fields, $example, $index); 
    }
    else { return '';}
}

sub record_count{
# ---------------------------------------------------
# Counts to total number of records/rows in the table 
# specified.

    my ($sth, $rows, $query);

    my $tablename = shift;

    $query = "SELECT COUNT(*) FROM $tablename";
    $sth = &exec_query($query) or return;
    $rows = $sth->fetchrow();
    $sth->finish();
    return $rows;
}

sub get_col_type{   
# ---------------------------------------------------
# Gets the type each field/column of the table specified.

    my $table = shift;
    my ($sth, $query, @ary, @type_ary);

    $query = "DESCRIBE $table";
    $sth = &exec_query($query) or return;
    while (@ary = $sth->fetchrow_array()) {
        push (@type_ary, $ary[1]);
    }
    $sth->finish();
    return @type_ary;
}

sub get_pre_query{

	my $in = shift;
	my $i = $in->param('query_select') || '';

	if (not $i) {
		return 0;
	}
	else{
		return $in->cookie("query_$i");	
	}
}

sub do_login{
# ---------------------------------------------------
# Assign login info to cookies.

    my $in = shift;
    my ($db_host, $db_user, $db_pass) = &assign_auth_cookie($in);

# redirects the user to the database list if init_login flag is on.
    if ($in->param('init_login')) {
        my $redirect_url = $CONFIG{'script_url'} . '?do='. $in->param('do') . '&data_source=' . $in->param('data_source'). '&init_login=' . $in->param('init_login'); 
        print $in->redirect(-url     => $redirect_url,
                            -cookie  => [$db_host, $db_user, $db_pass]);
        $PRINT_HEADER = 1;
        return 1;
    }
    else{ 
        print $in->header( -cookie => [$db_host, $db_user, $db_pass]); 
        $PRINT_HEADER = 1;
    } 
    
    return &html_back($in);
}

sub assign_auth_cookie{
# ---------------------------------------------------
# assign values to cookies used in the scirpt.

    my $in = shift;
    my $db_host = $in->param('db_host');
    my $db_user = $in->param('db_user');
    my $db_pass = $in->param('db_pass');
    my $expire  = $in->param('db_expire') ? "+5y" : "";

    my $host = $in->cookie( -name    => $CONFIG{'db_host_cookie_name'},
                            -value   => $db_host,
                            -path    => "/",
                            -expires => $expire
                            );

    my $user = $in->cookie( -name    => $CONFIG{'db_user_cookie_name'},
                            -value   => $db_user,
                            -path    => "/",
                            -expires => $expire
                            );

    my $pass = $in->cookie( -name    => $CONFIG{'db_pass_cookie_name'},
                            -value   => $db_pass,
                            -path    => "/",
                            -expires => $expire
                            );
    
    return ($host, $user, $pass);
}

sub assign_cookies{
# ---------------------------------------------------
# assign the current url to cookie.  This function tells
# the script where to go to after each login.  Also, if a 
# query is entered in the SQL monitor, the query will be saved
# as a cookie if requested.

    my $in = shift;
    my $cur_url;
    if (defined $ENV{REQUEST_METHOD} and (uc $ENV{REQUEST_METHOD} ne 'POST')) {
        $cur_url = $in->url;
    }
    else {
        $cur_url = $in->url ( query_string => 0 );
    }
    my $url_modified     = 0;
    my $data_source      = $in->param('data_source') || '';
    my $table            = $in->param('table')       || '';
    my $action           = $in->param('action')      || ''; 
    my $new_do           = '';
    my $new_action       = '';
	my $query			 = $in->param('query') || '';
	my $query_esc		 = $in->escape($query);

    if ($in->param('do') eq 'database') { 
        if ($in->param('comfirmed') || ($action eq 'create_db')) {
            $data_source = ''; $new_do = ''; $url_modified = 1;
        }
    }
    elsif ($in->param('do') eq 'insert_record') { $new_do = "$CONFIG{insert_origin}"; $url_modified = 1;}
    elsif ($in->param('do') eq 'modify') { if ($in->param('comfirmed')) { $new_do = 'tables'; $url_modified = 1;} }
    elsif ($in->param('do') eq 'create_table') {$new_do = 'tables'; $url_modified = 1;}
    elsif ($in->param('do') eq 'alter_table'){
        if (($action ne 'alter_col') && ($action ne 'do_alter_col')){
            $new_do = 'property'; $url_modified = 1;
        }
    }
    elsif ($in->param('do') eq 'sql_monitor' || $in->param('do') eq 'sql_monitor_file') {
        $new_do = 'top_level_op'; $new_action= 'sql_monitor'; $url_modified = 1;
    }
    elsif ($in->param('do') eq 'import') {
        $new_do = 'tables'; $url_modified = 1;
    }
    elsif ($in->param('do') eq 'export') {
        $new_do = 'tables'; $url_modified = 1;
    }
    elsif ($in->param('do') eq 'mysqldump') {
        $new_do = 'tables'; $url_modified = 1;
    }
    else { # do nothing 
    }

	

    if ($url_modified) {
        $cur_url = $CONFIG{'script_url'}. "?data_source=$data_source&do=$new_do&action=$new_action&table=$table";
    }
    my $url = $in->cookie(  -name    => $CONFIG{'url_cookie_name'},
                            -value   => $cur_url,
							-path    => "/"
    );

# store URL to cookie.  Save query to cookie if needed.
	if ($in->param('from_monitor')) {
		if ($in->param('save_query')) {
			if ($in->cookie('query_10')) { # delete the oldest query
				my $query_1 = $in->cookie( -name => 'query_1', -value => $in->cookie('query_2'), -path => "/", -expires => '+1y');
				my $query_2 = $in->cookie( -name => 'query_2', -value => $in->cookie('query_3'), -path => "/", -expires => '+1y');
				my $query_3 = $in->cookie( -name => 'query_3', -value => $in->cookie('query_4'), -path => "/", -expires => '+1y');
				my $query_4 = $in->cookie( -name => 'query_4', -value => $in->cookie('query_5'), -path => "/", -expires => '+1y');
				my $query_5 = $in->cookie( -name => 'query_5', -value => $in->cookie('query_6'), -path => "/", -expires => '+1y');
				my $query_6 = $in->cookie( -name => 'query_6', -value => $in->cookie('query_7'), -path => "/", -expires => '+1y');
				my $query_7 = $in->cookie( -name => 'query_7', -value => $in->cookie('query_8'), -path => "/", -expires => '+1y');
				my $query_8 = $in->cookie( -name => 'query_8', -value => $in->cookie('query_9'), -path => "/", -expires => '+1y');
				my $query_9 = $in->cookie( -name => 'query_9', -value => $in->cookie('query_10'), -path => "/", -expires => '+1y');
				my $query_10 = $in->cookie( -name => 'query_10', -value => $in->param('query'), -path => "/", -expires => '+1y');


				my $redirect_url = $CONFIG{'script_url'} . '?do='. $in->param('do') . '&data_source=' . $in->param('data_source') . "&action=monitor&from_monitor=1&query=$query_esc"; 
		        print $in->redirect(-url     => $redirect_url,
                                    -cookie => [$url, $query_1, $query_2, $query_3, $query_4, $query_5, $query_6, $query_7, $query_8, $query_9, $query_10]);

			}
			else{ # add the query to the last slot.
				my $new_query;
				for my $i (1 .. 10) {
					if (not $in->cookie("query_$i")) {
						$new_query = $in->cookie( -name => "query_$i", -value => $in->param('query'), -path => "/", -expires => '+1y');	
						last;
					}
				}
				my $redirect_url = $CONFIG{'script_url'} . '?do='. $in->param('do') . '&data_source=' . $in->param('data_source') . "&action=monitor&from_monitor=1&query=$query_esc"; 
		        print $in->redirect(-url     => $redirect_url,
                                    -cookie => [$url, $new_query]);
				
			}
		}
		elsif ($in->param('delete_query')){

			my @not_deleted_queries;
			for my $i (1 .. 10) {
				if (!$in->param("delete_query_$i")) {
					if (!$in->cookie("query_$i")) {
						push (@not_deleted_queries, 0)
					}
					else{
						push (@not_deleted_queries, $in->cookie("query_$i"));
					}
				}
			}
			
			my ($query_1, $query_2, $query_3, $query_4, $query_5, $query_6, $query_7, $query_8, $query_9, $query_10);

			if ($not_deleted_queries[0]) {$query_1 = $in->cookie( -name => 'query_1', -value => $not_deleted_queries[0], -path => "/", -expires => '+1y');}
			else {$query_1 = $in->cookie( -name => 'query_1', -value => '', -path => "/", -expires => '-1y');}
			if ($not_deleted_queries[1]) {$query_2 = $in->cookie( -name => 'query_2', -value => $not_deleted_queries[1], -path => "/", -expires => '+1y');}
			else {$query_2 = $in->cookie( -name => 'query_2', -value => '', -path => "/", -expires => '-1y');}
			if ($not_deleted_queries[2]) {$query_3 = $in->cookie( -name => 'query_3', -value => $not_deleted_queries[2], -path => "/", -expires => '+1y');}
			else {$query_3 = $in->cookie( -name => 'query_3', -value => '', -path => "/", -expires => '-1y');}
			if ($not_deleted_queries[3]) {$query_4 = $in->cookie( -name => 'query_4', -value => $not_deleted_queries[3], -path => "/", -expires => '+1y');}
			else {$query_4 = $in->cookie( -name => 'query_4', -value => '', -path => "/", -expires => '-1y');}
			if ($not_deleted_queries[4]) {$query_5 = $in->cookie( -name => 'query_5', -value => $not_deleted_queries[4], -path => "/", -expires => '+1y');}
			else {$query_5 = $in->cookie( -name => 'query_5', -value => '', -path => "/", -expires => '-1y');}
			if ($not_deleted_queries[5]) {$query_6 = $in->cookie( -name => 'query_6', -value => $not_deleted_queries[5], -path => "/", -expires => '+1y');}
			else {$query_6 = $in->cookie( -name => 'query_6', -value => '', -path => "/", -expires => '-1y');}
			if ($not_deleted_queries[6]) {$query_7 = $in->cookie( -name => 'query_7', -value => $not_deleted_queries[6], -path => "/", -expires => '+1y');}
			else {$query_7 = $in->cookie( -name => 'query_7', -value => '', -path => "/", -expires => '-1y');}
			if ($not_deleted_queries[7]) {$query_8 = $in->cookie( -name => 'query_8', -value => $not_deleted_queries[7], -path => "/", -expires => '+1y');}
			else {$query_8 = $in->cookie( -name => 'query_8', -value => '', -path => "/", -expires => '-1y');}
			if ($not_deleted_queries[8]) {$query_9 = $in->cookie( -name => 'query_9', -value => $not_deleted_queries[8], -path => "/", -expires => '+1y');}
			else {$query_9 = $in->cookie( -name => 'query_9', -value => '', -path => "/", -expires => '-1y');}
			if ($not_deleted_queries[9]) {$query_10 = $in->cookie( -name => 'query_10', -value => $not_deleted_queries[9], -path => "/", -expires => '+1y');}
			else {$query_10 = $in->cookie( -name => 'query_10', -value => '', -path => "/", -expires => '-1y');}

			my $redirect_url = $CONFIG{'script_url'} . '?do='. $in->param('do') . '&data_source=' . $in->param('data_source'); 
	        print $in->redirect(-url     => $redirect_url,
                                -cookie  => [$url, $query_1, $query_2, $query_3, $query_4, $query_5, $query_6, $query_7, $query_8, $query_9, $query_10]);

		}
		else{
			print $in->header(-cookie => $url);
		}
	}
	else{
		print $in->header(-cookie => $url);
	}
    $PRINT_HEADER=1;

    return 1;
}

sub do_logout{
# ---------------------------------------------------
# Logs the users out by making all the cookies used in 
# the script expire.

    my $in = shift;
    my ($host, $user, $pass, $url);
    
    ($host, $user, $pass, $url) = &cookie_cleanup($in);
    print $in->header(-cookie => [$host, $user, $pass, $url]);
    $PRINT_HEADER=1;
    return &html_logout;
}

sub cookie_cleanup{
# ---------------------------------------------------
# Makes all cookies expire.

    my $in = shift;

    my $host = $in->cookie( -name    => $CONFIG{'db_host_cookie_name'},
                            -value   => '',
                            -expires => '-1y',
                            -path    => '/');
    my $user = $in->cookie( -name    => $CONFIG{'db_user_cookie_name'},
                            -value   => '',
                            -expires => '-1y',
                            -path    => '/');
    my $pass = $in->cookie( -name    => $CONFIG{'db_pass_cookie_name'},
                            -value   => '',
                            -expires => '-1y',
                            -path    => '/');
    my $url = $in->cookie(  -name    => $CONFIG{'url_cookie_name'},
                            -value   => '',
                            -expires => '-1y',
                            -path    => '/');
    return ($host, $user, $pass, $url);
}


sub connect_db{
# ---------------------------------------------------
# Tries to connect the database with user name and password 
# provided first.  If access denied then tries connecting 
# again with user name and password undefined.  If both 
# fail then an login page will be prompted.

    my $in = shift;
    my ($db, $db_host, $db_port, $username, $password, $message);
    my $data_source = $in->param("data_source") || 'DBI:mysql:';
    my $init_login  = $in->param('init_login');
    my $cur_db      = &get_db($in->param('data_source')) || '',

    # gets user data from cookies.
    $username = $in->cookie($CONFIG{'db_user_cookie_name'}) || '';
    $password = $in->cookie($CONFIG{'db_pass_cookie_name'}) || '';
    $db_host  = $in->cookie($CONFIG{'db_host_cookie_name'}) || 'localhost';

    my $db_connection = $in->param("db_for_connection") || '';

    if ($db_connection) { 
        $data_source = "DBI:mysql:" . $in->param("db_for_connection"); 
    }

# while under demo mode, check if the user is using the host specified and make sure
# the script only connect to the database specified.


    if ($db_host && ($data_source eq 'DBI:mysql:')) { $data_source = "DBI:mysql:host=$db_host"}
    else { $data_source = $data_source . ":$db_host" }

    # set connect info to the info specified in the config file if direct_connect is on.
    if ($CONFIG{'direct_connect'}) {
        $db       = $CONFIG{'direct_db'};
        $username = $CONFIG{'direct_user'};
        $password = $CONFIG{'direct_pass'};
        $db_host  = $CONFIG{'direct_host'};
        $db_port  = $CONFIG{'direct_port'};

        $data_source = "DBI:mysql:$db";
        if ($db_host) { $data_source .= ":$db_host"; }
        if ($db_port) { $data_source .= ":$db_port"; }
    }

# while under demo mode, check if the user is using the database specified.

    # connects to mysql.
    $DBH = DBI->connect("$data_source", "$username", "$password", { RaiseError => 0, PrintError => 0, AutoCommit => 1 });

    if (not $DBH){
        my $orig_error = $DBI::errstr;

# If the connection fails then the user name and/or the password may be wrong or that they are correct but the 
# the connection fail for some other reason

        if ($DBI::errstr =~ m/(.ccess denied)/) { # If the user name and/or the password is incorrect
            $DBH = DBI->connect("$data_source", undef, undef, { RaiseError => 0, PrintError => 0, AutoCommit => 1 })
                        or return &sqlerr($orig_error);
        }
        elsif ($DBI::errstr =~ m/(.nknown database)/) { # database specified not existed
            return &sqlerr("$DBI::errstr."); 
        }
        elsif (($DBI::errstr =~ m/an't connect to/) || ($DBI::errstr =~ m/Unknown MySQL Server Host/) ) {
            return &sqlerr("$DBI::errstr.");
        }
        else { &cgierr("connection failed: $DBI::errstr"); }
    }
    return 1;
}

sub exec_query{
# ---------------------------------------------------
# Send the input qeury MySQL thru database handler.

    my $query = shift;
    my ($sth);
    
    $sth = $DBH->prepare($query) or return &sqlerr("$DBI::errstr. <P>Query: $query");
    $sth->execute() or return &sqlerr("$DBI::errstr. <P>Query: $query"); 

    return $sth;
}

sub sqlerr{
# ---------------------------------------------------
# Error prompt.

    my $in = new GT::CGI;
    my $error = shift;
    my ($message, $init_login);

    $init_login = 0;
    my $db_connection = $in->param("db_for_connection") || '';

    # If access denied, then a login page will be displayed.
    if ($error =~ m/(.ccess denied)/){ 
        $message = "<p>Permission to perform action denied!</p><blockquote>MySQL Said: $error</blockquote><p>Please enter your user name and password";
        my $args = $in->get_hash;
        if (! keys %$args or ($in->param('do') eq 'login')) {
            $message = 'Welcome! Please enter your log-in info.'; 
            $init_login = 1;    
        }
        elsif ($in->param('init_login')){ 
            $message = 'Login failed.  Please enter another hostname/username/password.'; 
            $init_login = 1;
        }
        &html_login ($in, $message, $init_login); 
        if ( $CONFIG{'debug'} ) {&cgierr("debug");}
        return;
    }

    # If connect error, login page is prompted to let the user to enter another 
    # host name.
    elsif (($DBI::errstr =~ m/an't connect to/) || ($DBI::errstr =~ m/Unknown MySQL Server Host/) ) {
        $message = qq~Connection to MySQL failed.
                      The hostname may be different or the server may be down.
                      Please enter a new hostname and try again.~;
        my $args = $in->get_hash;
        if (! keys %$args or ($in->param('do') eq 'login')) {
            $message = 'Welcome! Please enter your log-in info.'; 
            $init_login = 1;    
        }
        elsif ($in->param('init_login')){ 
            $message = 'Login failed.  Please enter another hostname/username/password.';
            $init_login = 1;
        }
        &html_login($in, $message, $init_login);
        if ( $CONFIG{'debug'} ) {&cgierr("debug");}
        return;
    }

    # database specified not existed
    elsif ($DBI::errstr =~ m/(.nknown database)/) { 
        if (!$db_connection) {
            $message = "Connection to MySQL failed.  A database name is needed for connection.";
            &html_login_dbname($in, $message, $init_login);
            if ( $CONFIG{'debug'} ) {&cgierr("debug");}
            return;
        }
        else{
            $message = "Unknown database '$db_connection'.  Please enter another database name.";
            &html_login_dbname($in, $message, $init_login);
            if ( $CONFIG{'debug'} ) {&cgierr("debug");}
            return;
        }
    }

    # display the error message.
    else{
        if ($CONFIG{'debug'}) { 
            &cgierr($error);
        }
        else { 
            html_sqlerr($in, $error); 
            if ( $CONFIG{'debug'} ) {&cgierr("debug");}
            return;
        }
    }
}

sub cgierr{
# --------------------------------------------------------
# Displays any errors and prints out FORM and ENVIRONMENT
# information. Useful for debugging.
#
    my $in;
    eval { $in = new GT::CGI; };
    if (defined $GT::CGI::PRINTED_HEAD) {
        print "Content-type: text/html\n\n" unless ($GT::CGI::PRINTED_HEAD);
    }
    else {
        print "Content-type: text/html\n\n";
    }

    my ($key, $env);
    my ($error, $nolog) = @_;
    print "</td></tr></table>";
    print "</td></tr></table></center></center>";
    if ($CONFIG{'debug'}) {print "<PRE>\n\nDEBUG\n==========================================\n";}
    else {print "<PRE>\n\nCGI ERROR\n==========================================\n";}
    $error    and print "Error Message       : $error\n";
    $0        and print "Script Location     : $0\n";
    $]        and print "Perl Version        : $]\n";
    
    print "\nConfiguration\n-------------------------------------------\n";
    foreach $key (sort keys %CONFIG) {
        my $space = " " x (20 - length($key));
        print "$key$space: $CONFIG{$key}\n";
    }
    if ($in) {
        print "\nCookies\n-------------------------------------------\n";
        print "$CONFIG{'db_user_cookie_name'} : " . $in->cookie($CONFIG{'db_user_cookie_name'});
        print "\n$CONFIG{'db_pass_cookie_name'} : " . $in->cookie($CONFIG{'db_pass_cookie_name'});
        print "\n$CONFIG{'db_host_cookie_name'} : " . $in->cookie($CONFIG{'db_host_cookie_name'});
        print "\n$CONFIG{'url_cookie_name'} :   " . $in->cookie($CONFIG{'url_cookie_name'});
        print "\nquery_1 :   " . $in->cookie('query_1');
        print "\nquery_2 :   " . $in->cookie('query_2');
        print "\nquery_3 :   " . $in->cookie('query_3');
        print "\nquery_4 :   " . $in->cookie('query_4');
        print "\nquery_5 :   " . $in->cookie('query_5');
        print "\nquery_6 :   " . $in->cookie('query_6');
        print "\nquery_7 :   " . $in->cookie('query_7');
        print "\nquery_8 :   " . $in->cookie('query_8');
        print "\nquery_9 :   " . $in->cookie('query_9');
        print "\nquery_10 :   " . $in->cookie('query_10');

        print "\n\nForm Variables\n-------------------------------------------\n";
        foreach $key (sort $in->param) {
           my $space = " " x (20 - length($key));
           print "$key$space: " . $in->param($key) . "\n";
        }
    }
    print "\nEnvironment Variables\n-------------------------------------------\n";
    foreach $env (sort keys %ENV) {
        my $space = " " x (20 - length($env));
        print "$env$space: $ENV{$env}\n";
    }
    print "\nStack Trace \n-------------------------------------------\n";
    my $i = 0;
    while (my ($file, $line, $sub) = (caller($i++))[1,2,3]) {
        print qq!($sub) called from ($file) line ($line)<BR>\n!;
    }
    print "\n</PRE>";
    exit;
}

#=================================================#
#           Check Required Libraries              #
#            -- w/ user friendly error message    #
#=================================================#

sub check_libraries{
# --------------------------------------------------------
# Required Librariers
# Make sure we are using perl 5.003 and load other required files.

    eval { 
        require DBI;
    };
    if ($@) { 
        my $message = "<font size=+2>Error loading required libraries</font>\n\nReason: $@\n\n1.\tIt's likely that you do not have the module installed.\n\tYou can find this at: <a href=http://www.perl.com/CPAN/modules/by-module/DBI/>http://www.perl.com/CPAN/modules/by-module/DBI/</a>\n\n2.\tIf the file exists:\n\t";
        $message .= qq~Please check if the path to the folder that contains the file is in \@INC.
        
\t\@INC is a special variable in Perl that contains the list of places to
\tlook for Perl scripts to be evaluated by the do, require, or use
\tconstructs.  The error occurs because DBI.pm cannot be found in \@INC.  
\tPlease check if you have Perl installed properly and check if you have the
\tcorrect path to perl in the cgi.  If you do, then please contact the
\tserver admin.
~;
        library_error($message); 
    } 

    eval {
        require DBD::mysql;
    };
    if ($@) { 
        my $message = "<font size=+2>Error loading required libraries</font>\n\nReason: $@\n\n1.\tIt's likely that you do not have the module installed.\n\tYou can find this at: <a href=http://www.perl.com/CPAN/modules/by-module/DBD/>http://www.perl.com/CPAN/modules/by-module/DBD/</a>\n\n2.\tIf the file exists:\n\t";
        $message .= qq~Please check if the path to the folder that contains the file is in \@INC.
        
\t\@INC is a special variable in Perl that contains the list of places to
\tlook for Perl scripts to be evaluated by the do, require, or use
\tconstructs.  The error occurs because mysql.pm cannot be found in \@INC.  
\tPlease check if you have Perl installed properly and check if you have the
\tcorrect path to perl in the cgi.  If you do, then please contact the
\tserver admin.
~;
        library_error($message); 
    } 
    eval {
        require strict;
        require 5.004;          # We need at least Perl 5.004
    };
    if ($@) { &cgierr("MySQLMan requires at least perl version 5.004 or better. You are using: $[. ($@)"); }

# Try and load config file.
    eval { require 'mysql.cfg'; };
    if ($@) {
        my $err = $@; local ($@);
        eval { 
            require FindBin;
            $SCRIPT_PATH = $FindBin::Bin || $FindBin::Bin; # Silence -w
            unshift @INC, $SCRIPT_PATH;
            require 'mysql.cfg';
        };
        if ($@) {
            cgierr("MySQLMan can not find the mysql.cfg config file. Please edit mysql.cgi and set \$SCRIPT_PATH to the directory where MySQLMan resides. Error Message was: $@");
        }
    }
    eval {
        require GT::CGI;
        require GT::Template;
        require 'html.pl';
        require 'mysql.cfg';
    };
    if ($@) {
        library_error("<font size=+2>Error loading required libraries</font>\n\nReason: $@\n\n1.\tPlease check if the file is in MySQLMan directory, permission is set correctly, and that it compiles.\n\n2.\tIf the file exists in the folder, then check if the path to MySQLMan folder is in \@INC.\n\tIf \@INC does not contain the path then add\n\t\t<B>use lib '/path/to/MySQLMan';</B>\n\tto the beginning of mysql.cgi.\n\n\tPlease refer the comments in mysql.cgi for detail.\n\n3.\tPlease check if the file was uploaded in <B>ASCII</B> mode (instead of BINARY mode).");
    }
}


sub library_error{
# --------------------------------------------------------
# HTML error page.  Displayed when there error occurred while 
# loading required libraries.

    my ($message) = @_;    
    if (defined $GT::CGI::PRINTED_HEAD) {
        print "Content-type: text/html\n\n" unless ($GT::CGI::PRINTED_HEAD);
    }
    else {
        print "Content-type: text/html\n\n";
    }
    print qq~<HTML>
<HEAD>
<TITLE>
MySQLMan - Error loading required libraries.
</TITLE>
</HEAD>

<BODY BGCOLOR="#CCCCCC">

<table border=1 bgcolor="#FFFFFF" cellpadding=5 cellspacing=3 width="100%" align=center valign=top>
    <tr>
        <td bgcolor="navy">
          <FONT FACE="MS Sans Serif, arial,helvetica" size=1 COLOR="#FFFFFF">   
          <b>MySQLMan: Error loading required libraries.</b>
          </font>
        </td>
    </tr>
    <tr>
        <td>
            <UL>
            <pre>
            <font FACE="MS Sans Serif, arial,helvetica">
$message
            </font>
            </pre>
            </UL>
        </td>
    </tr>
</table>

</BODY>
</HTML>

~;

exit;
}
